@@ -176,7 +176,37 @@ evas_object_clip_set(Evas_Object *obj, Evas_Object *clip)
    return;
    MAGIC_CHECK_END();
    if (obj->cur.clipper == clip) return;
-   if (obj == clip) return;
+   if (obj == clip)
+     {
+        CRIT("Setting clip %p on itself", obj);
+        return;
+     }
+   if (clip->delete_me)
+     {
+        CRIT("Setting deleted object %p as clip obj %p", clip, obj);
+        abort();
+        return;
+     }
+   if (obj->delete_me)
+     {
+        CRIT("Setting object %p as clip to deleted obj %p", clip, obj);
+        abort();
+        return;
+     }
+   if (!obj->layer)
+     {
+        CRIT("No evas surface associated with object (%p)", obj);
+        abort();
+        return;
+     }
+   if ((obj->layer && clip->layer) &&
+       (obj->layer->evas != clip->layer->evas))
+     {
+        CRIT("Setting object %p from Evas (%p) to another Evas (%p)", obj, obj->layer->evas, clip->layer->evas);
+        abort();
+        return;
+     }
+
    if (evas_object_intercept_call_clip_set(obj, clip)) return;
    // illegal to set anything but a rect as a clip
    if (clip->type != o_rect_type)
@@ -219,11 +249,16 @@ evas_object_clip_set(Evas_Object *obj, Evas_Object *clip)
      }
    obj->cur.clipper = clip;
    clip->clip.clipees = eina_list_append(clip->clip.clipees, obj);
-   if (clip->clip.clipees) clip->cur.have_clipees = 1;
+   if (clip->clip.clipees)
+     {
+        clip->cur.have_clipees = 1;
+        if (clip->changed)
+          evas_object_update_bounding_box(clip);
+     }
 
    /* If it's NOT a rectangle set the mask bits too */
    /* FIXME: Optmz ths chck */
-   if (strcmp(evas_object_type_get(clip),"rectangle") == 0)
+   if (clip->type == o_rect_type)
       obj->cur.mask = NULL;
    else
      {
