@@ -9,7 +9,6 @@ static void evas_object_intercept_deinit(Evas_Object *obj);
 static void
 evas_object_intercept_init(Evas_Object *obj)
 {
-   /* MEM OK */
    if (!obj->interceptors)
      obj->interceptors = evas_mem_calloc(sizeof(Evas_Intercept_Func));
 }
@@ -17,7 +16,6 @@ evas_object_intercept_init(Evas_Object *obj)
 static void
 evas_object_intercept_deinit(Evas_Object *obj)
 {
-   /* MEM OK */
    if (!obj->interceptors) return;
    if ((obj->interceptors->show.func) ||
        (obj->interceptors->hide.func) ||
@@ -41,227 +39,194 @@ evas_object_intercept_deinit(Evas_Object *obj)
 void
 evas_object_intercept_cleanup(Evas_Object *obj)
 {
-   /* MEM OK */
    if (obj->interceptors) free(obj->interceptors);
 }
 
 int
 evas_object_intercept_call_show(Evas_Object *obj)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->show.func) != NULL);
-   if (obj->interceptors->show.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->show.func);
+   if (ret)
      obj->interceptors->show.func(obj->interceptors->show.data, obj);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_hide(Evas_Object *obj)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->hide.func) != NULL);
-   if (obj->interceptors->hide.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->hide.func);
+   if (ret)
      obj->interceptors->hide.func(obj->interceptors->hide.data, obj);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->move.func) != NULL);
-   if (obj->interceptors->move.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->move.func);
+   if (ret)
      obj->interceptors->move.func(obj->interceptors->move.data, obj, x, y);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->resize.func) != NULL);
-   if (obj->interceptors->resize.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->resize.func);
+   if (ret)
      obj->interceptors->resize.func(obj->interceptors->resize.data, obj, w, h);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_raise(Evas_Object *obj)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->raise.func) != NULL);
-   if (obj->interceptors->raise.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->raise.func);
+   if (ret)
      obj->interceptors->raise.func(obj->interceptors->raise.data, obj);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_lower(Evas_Object *obj)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->lower.func) != NULL);
-   if (obj->interceptors->lower.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->lower.func);
+   if (ret)
      obj->interceptors->lower.func(obj->interceptors->lower.data, obj);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_stack_above(Evas_Object *obj, Evas_Object *above)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->stack_above.func) != NULL);
-   if (obj->interceptors->stack_above.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->stack_above.func);
+   if (ret)
      obj->interceptors->stack_above.func(obj->interceptors->stack_above.data, obj, above);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_stack_below(Evas_Object *obj, Evas_Object *below)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->stack_below.func) != NULL);
-   if (obj->interceptors->stack_below.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->stack_below.func);
+   if (ret)
      obj->interceptors->stack_below.func(obj->interceptors->stack_below.data, obj, below);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_layer_set(Evas_Object *obj, int l)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->layer_set.func) != NULL);
-   if (obj->interceptors->layer_set.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->layer_set.func);
+   if (ret)
      obj->interceptors->layer_set.func(obj->interceptors->layer_set.data, obj, l);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_color_set(Evas_Object *obj, int r, int g, int b, int a)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->color_set.func) != NULL);
-   if (obj->interceptors->color_set.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->color_set.func);
+   if (ret)
      obj->interceptors->color_set.func(obj->interceptors->color_set.data, obj, r, g, b, a);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_clip_set(Evas_Object *obj, Evas_Object *clip)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->clip_set.func) != NULL);
-   if (obj->interceptors->clip_set.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->clip_set.func);
+   if (ret)
      obj->interceptors->clip_set.func(obj->interceptors->clip_set.data, obj, clip);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 int
 evas_object_intercept_call_clip_unset(Evas_Object *obj)
 {
-   /* MEM OK */
    int ret;
 
    if (!obj->interceptors) return 0;
    if (obj->intercepted) return 0;
-   obj->intercepted = 1;
-   ret = ((obj->interceptors->clip_unset.func) != NULL);
-   if (obj->interceptors->clip_unset.func)
+   obj->intercepted = EINA_TRUE;
+   ret = !!(obj->interceptors->clip_unset.func);
+   if (ret)
      obj->interceptors->clip_unset.func(obj->interceptors->clip_unset.data, obj);
-   obj->intercepted = 0;
+   obj->intercepted = EINA_FALSE;
    return ret;
 }
 
 /* public calls */
 
-/**
- * @addtogroup Evas_Object_Group_Interceptors
- * @{
- */
-
-/**
- * Set the callback function that intercepts a show event of a object.
- *
- * @param obj The given canvas object pointer.
- * @param func The given function to be the callback function.
- * @param data The data passed to the callback fucntion.
- *
- * This function sets a callback function to intercepts a show event
- * of a canvas object.
- *
- * @see evas_object_intercept_show_callback_del().
- *
- */
 EAPI void
-evas_object_intercept_show_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj), const void *data)
+evas_object_intercept_show_callback_add(Evas_Object *obj, Evas_Object_Intercept_Show_Cb func, const void *data)
 {
-   /* MEM OK */
-
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -272,23 +237,9 @@ evas_object_intercept_show_callback_add(Evas_Object *obj, void (*func) (void *da
    obj->interceptors->show.data = (void *)data;
 }
 
-/**
- * Unset the callback function that intercepts a show event of a
- * object.
- *
- * @param obj The given canvas object pointer.
- * @param func The given callback function.
- *
- * This function sets a callback function to intercepts a show event
- * of a canvas object.
- *
- * @see evas_object_intercept_show_callback_add().
- *
- */
 EAPI void *
-evas_object_intercept_show_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj))
+evas_object_intercept_show_callback_del(Evas_Object *obj, Evas_Object_Intercept_Show_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -303,23 +254,9 @@ evas_object_intercept_show_callback_del(Evas_Object *obj, void (*func) (void *da
    return data;
 }
 
-/**
- * Set the callback function that intercepts a hide event of a object.
- *
- * @param obj The given canvas object pointer.
- * @param func The given function to be the callback function.
- * @param data The data passed to the callback fucntion.
- *
- * This function sets a callback function to intercepts a hide event
- * of a canvas object.
- *
- * @see evas_object_intercept_hide_callback_del().
- *
- */
 EAPI void
-evas_object_intercept_hide_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj), const void *data)
+evas_object_intercept_hide_callback_add(Evas_Object *obj, Evas_Object_Intercept_Hide_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -330,23 +267,9 @@ evas_object_intercept_hide_callback_add(Evas_Object *obj, void (*func) (void *da
    obj->interceptors->hide.data = (void *)data;
 }
 
-/**
- * Unset the callback function that intercepts a hide event of a
- * object.
- *
- * @param obj The given canvas object pointer.
- * @param func The given callback function.
- *
- * This function sets a callback function to intercepts a hide event
- * of a canvas object.
- *
- * @see evas_object_intercept_hide_callback_add().
- *
- */
 EAPI void *
-evas_object_intercept_hide_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj))
+evas_object_intercept_hide_callback_del(Evas_Object *obj, Evas_Object_Intercept_Hide_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -361,23 +284,9 @@ evas_object_intercept_hide_callback_del(Evas_Object *obj, void (*func) (void *da
    return data;
 }
 
-/**
- * Set the callback function that intercepts a move event of a object.
- *
- * @param obj The given canvas object pointer.
- * @param func The given function to be the callback function.
- * @param data The data passed to the callback fucntion.
- *
- * This function sets a callback function to intercepts a move event
- * of a canvas object.
- *
- * @see evas_object_intercept_move_callback_del().
- *
- */
 EAPI void
-evas_object_intercept_move_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Coord x, Evas_Coord y), const void *data)
+evas_object_intercept_move_callback_add(Evas_Object *obj, Evas_Object_Intercept_Move_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -388,23 +297,9 @@ evas_object_intercept_move_callback_add(Evas_Object *obj, void (*func) (void *da
    obj->interceptors->move.data = (void *)data;
 }
 
-/**
- * Unset the callback function that intercepts a move event of a
- * object.
- *
- * @param obj The given canvas object pointer.
- * @param func The given callback function.
- *
- * This function sets a callback function to intercepts a move event
- * of a canvas object.
- *
- * @see evas_object_intercept_move_callback_add().
- *
- */
 EAPI void *
-evas_object_intercept_move_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Coord x, Evas_Coord y))
+evas_object_intercept_move_callback_del(Evas_Object *obj, Evas_Object_Intercept_Move_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -419,16 +314,9 @@ evas_object_intercept_move_callback_del(Evas_Object *obj, void (*func) (void *da
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_resize_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Coord w, Evas_Coord h), const void *data)
+evas_object_intercept_resize_callback_add(Evas_Object *obj, Evas_Object_Intercept_Resize_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -439,16 +327,9 @@ evas_object_intercept_resize_callback_add(Evas_Object *obj, void (*func) (void *
    obj->interceptors->resize.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_resize_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Coord w, Evas_Coord h))
+evas_object_intercept_resize_callback_del(Evas_Object *obj, Evas_Object_Intercept_Resize_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -463,16 +344,9 @@ evas_object_intercept_resize_callback_del(Evas_Object *obj, void (*func) (void *
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_raise_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj), const void *data)
+evas_object_intercept_raise_callback_add(Evas_Object *obj, Evas_Object_Intercept_Raise_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -483,16 +357,9 @@ evas_object_intercept_raise_callback_add(Evas_Object *obj, void (*func) (void *d
    obj->interceptors->raise.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_raise_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj))
+evas_object_intercept_raise_callback_del(Evas_Object *obj, Evas_Object_Intercept_Raise_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -507,16 +374,9 @@ evas_object_intercept_raise_callback_del(Evas_Object *obj, void (*func) (void *d
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_lower_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj), const void *data)
+evas_object_intercept_lower_callback_add(Evas_Object *obj, Evas_Object_Intercept_Lower_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -527,16 +387,9 @@ evas_object_intercept_lower_callback_add(Evas_Object *obj, void (*func) (void *d
    obj->interceptors->lower.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_lower_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj))
+evas_object_intercept_lower_callback_del(Evas_Object *obj, Evas_Object_Intercept_Lower_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -551,16 +404,9 @@ evas_object_intercept_lower_callback_del(Evas_Object *obj, void (*func) (void *d
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_stack_above_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Object *above), const void *data)
+evas_object_intercept_stack_above_callback_add(Evas_Object *obj, Evas_Object_Intercept_Stack_Above_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -571,16 +417,9 @@ evas_object_intercept_stack_above_callback_add(Evas_Object *obj, void (*func) (v
    obj->interceptors->stack_above.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_stack_above_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Object *above))
+evas_object_intercept_stack_above_callback_del(Evas_Object *obj, Evas_Object_Intercept_Stack_Above_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -595,16 +434,9 @@ evas_object_intercept_stack_above_callback_del(Evas_Object *obj, void (*func) (v
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_stack_below_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Object *below), const void *data)
+evas_object_intercept_stack_below_callback_add(Evas_Object *obj, Evas_Object_Intercept_Stack_Below_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -615,16 +447,9 @@ evas_object_intercept_stack_below_callback_add(Evas_Object *obj, void (*func) (v
    obj->interceptors->stack_below.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_stack_below_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Object *below))
+evas_object_intercept_stack_below_callback_del(Evas_Object *obj, Evas_Object_Intercept_Stack_Below_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -639,16 +464,9 @@ evas_object_intercept_stack_below_callback_del(Evas_Object *obj, void (*func) (v
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_layer_set_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, int l), const void *data)
+evas_object_intercept_layer_set_callback_add(Evas_Object *obj, Evas_Object_Intercept_Layer_Set_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -659,16 +477,9 @@ evas_object_intercept_layer_set_callback_add(Evas_Object *obj, void (*func) (voi
    obj->interceptors->layer_set.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_layer_set_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, int l))
+evas_object_intercept_layer_set_callback_del(Evas_Object *obj, Evas_Object_Intercept_Layer_Set_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -683,16 +494,9 @@ evas_object_intercept_layer_set_callback_del(Evas_Object *obj, void (*func) (voi
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_color_set_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, int r, int g, int b, int a), const void *data)
+evas_object_intercept_color_set_callback_add(Evas_Object *obj, Evas_Object_Intercept_Color_Set_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -703,16 +507,9 @@ evas_object_intercept_color_set_callback_add(Evas_Object *obj, void (*func) (voi
    obj->interceptors->color_set.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_color_set_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, int r, int g, int b, int a))
+evas_object_intercept_color_set_callback_del(Evas_Object *obj, Evas_Object_Intercept_Color_Set_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -727,16 +524,9 @@ evas_object_intercept_color_set_callback_del(Evas_Object *obj, void (*func) (voi
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_clip_set_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Object *clip), const void *data)
+evas_object_intercept_clip_set_callback_add(Evas_Object *obj, Evas_Object_Intercept_Clip_Set_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -747,16 +537,9 @@ evas_object_intercept_clip_set_callback_add(Evas_Object *obj, void (*func) (void
    obj->interceptors->clip_set.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_clip_set_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj, Evas_Object *clip))
+evas_object_intercept_clip_set_callback_del(Evas_Object *obj, Evas_Object_Intercept_Clip_Set_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -771,16 +554,9 @@ evas_object_intercept_clip_set_callback_del(Evas_Object *obj, void (*func) (void
    return data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void
-evas_object_intercept_clip_unset_callback_add(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj), const void *data)
+evas_object_intercept_clip_unset_callback_add(Evas_Object *obj, Evas_Object_Intercept_Clip_Unset_Cb func, const void *data)
 {
-   /* MEM OK */
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
@@ -791,16 +567,9 @@ evas_object_intercept_clip_unset_callback_add(Evas_Object *obj, void (*func) (vo
    obj->interceptors->clip_unset.data = (void *)data;
 }
 
-/**
- * To be documented.
- *
- * FIXME: To be fixed.
- *
- */
 EAPI void *
-evas_object_intercept_clip_unset_callback_del(Evas_Object *obj, void (*func) (void *data, Evas_Object *obj))
+evas_object_intercept_clip_unset_callback_del(Evas_Object *obj, Evas_Object_Intercept_Clip_Unset_Cb func)
 {
-   /* MEM OK */
    void *data;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -814,7 +583,3 @@ evas_object_intercept_clip_unset_callback_del(Evas_Object *obj, void (*func) (vo
    evas_object_intercept_deinit(obj);
    return data;
 }
-
-/**
- * @}
- */
