@@ -13,8 +13,10 @@ struct _Evas_Object_Smart
    Eina_Inlist      *contained;
    Evas_Smart_Cb_Description_Array callbacks_descriptions;
    int               walking_list;
+   int               member_count;
    Eina_Bool         deletions_waiting : 1;
    Eina_Bool         need_recalculate : 1;
+   Eina_Bool         update_boundingbox_needed : 1;
 };
 
 struct _Evas_Smart_Callback
@@ -94,9 +96,60 @@ evas_object_smart_data_get(const Evas_Object *obj)
    MAGIC_CHECK_END();
    o = (Evas_Object_Smart *)(obj->object_data);
    if (!o) return NULL;
+   if (o->magic != MAGIC_OBJ_SMART) return NULL;
    return o->data;
 }
 
+EAPI const void *
+evas_object_smart_interface_get(const Evas_Object *obj,
+                                const char *name)
+{
+   Evas_Smart *s;
+   unsigned int i;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return NULL;
+   MAGIC_CHECK_END();
+
+   s = evas_object_smart_smart_get(obj);
+   if (!s) return NULL;
+
+   for (i = 0; i < s->interfaces.size; i++)
+     {
+        const Evas_Smart_Interface *iface;
+
+        iface = s->interfaces.array[i];
+
+        if (iface->name == name)
+          return iface;
+     }
+
+   return NULL;
+}
+
+EAPI void *
+evas_object_smart_interface_data_get(const Evas_Object *obj,
+                                     const Evas_Smart_Interface *iface)
+{
+   unsigned int i;
+   Evas_Smart *s;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return NULL;
+   MAGIC_CHECK_END();
+
+   s = evas_object_smart_smart_get(obj);
+   if (!s) return NULL;
+
+   for (i = 0; i < s->interfaces.size; i++)
+     {
+        if (iface == s->interfaces.array[i])
+          return obj->interface_privates[i];
+     }
+
+   return NULL;
+}
+
 EAPI Evas_Smart *
 evas_object_smart_smart_get(const Evas_Object *obj)
 {
@@ -158,6 +211,7 @@ evas_object_smart_member_add(Evas_Object *obj, Evas_Object *smart_obj)
 
    if (obj->smart.parent) evas_object_smart_member_del(obj);
 
+   o->member_count++;
    evas_object_release(obj, 1);
    obj->layer = smart_obj->layer;
    obj->cur.layer = obj->layer->layer;
@@ -170,6 +224,7 @@ evas_object_smart_member_add(Evas_Object *obj, Evas_Object *smart_obj)
    evas_object_mapped_clip_across_mark(obj);
    if (smart_obj->smart.smart->smart_class->member_add)
      smart_obj->smart.smart->smart_class->member_add(smart_obj, obj);
+   evas_object_update_bounding_box(obj);
 }
 
 EAPI void
@@ -190,6 +245,7 @@ evas_object_smart_member_del(Evas_Object *obj)
 
    o = (Evas_Object_Smart *)(obj->smart.parent->object_data);
    o->contained = eina_inlist_remove(o->contained, EINA_INLIST_GET(obj));
+   o->member_count--;
    obj->smart.parent = NULL;
    evas_object_smart_member_cache_invalidate(obj, EINA_TRUE, EINA_TRUE);
    obj->layer->usage--;
@@ -219,6 +275,8 @@ evas_object_smart_type_check(const Evas_Object *obj, const char *type)
    return EINA_FALSE;
    MAGIC_CHECK_END();
 
+   EINA_SAFETY_ON_FALSE_RETURN_VAL(type, EINA_FALSE);
+
    if (!obj->smart.smart)
      return EINA_FALSE;
    sc = obj->smart.smart->smart_class;
@@ -241,6 +299,8 @@ evas_object_smart_type_check_ptr(const Evas_Object *obj, const char *type)
    return EINA_FALSE;
    MAGIC_CHECK_END();
 
+   EINA_SAFETY_ON_FALSE_RETURN_VAL(type, EINA_FALSE);
+
    if (!obj->smart.smart)
      return EINA_FALSE;
    sc = obj->smart.smart->smart_class;
@@ -264,6 +324,7 @@ evas_object_smart_members_get(const Evas_Object *obj)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
    MAGIC_CHECK_END();
+   if (!obj->smart.smart) return NULL;
    o = (Evas_Object_Smart *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
    return NULL;
@@ -284,6 +345,7 @@ evas_object_smart_members_get_direct(const Evas_Object *obj)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
    MAGIC_CHECK_END();
+   if (!obj->smart.smart) return NULL;
    o = (Evas_Object_Smart *)(obj->object_data);
    MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
    return NULL;
@@ -295,14 +357,84 @@ void
 _evas_object_smart_members_all_del(Evas_Object *obj)
 {
    Evas_Object_Smart *o = (Evas_Object_Smart *)(obj->object_data);
-   while (o->contained)
-     evas_object_del((Evas_Object *)(o->contained));
+   Evas_Object *memobj;
+   Eina_Inlist *itrn;
+   EINA_INLIST_FOREACH_SAFE(o->contained, itrn, memobj)
+     {
+        evas_object_del((Evas_Object *) memobj);
+     }
+}
+
+static void
+_evas_smart_class_ifaces_private_data_alloc(Evas_Object *obj,
+                                            Evas_Smart *s)
+{
+   unsigned int i, total_priv_sz = 0;
+   const Evas_Smart_Class *sc;
+   unsigned char *ptr;
+
+   /* get total size of interfaces private data */
+   for (sc = s->smart_class; sc; sc = sc->parent)
+     {
+        const Evas_Smart_Interface **ifaces_array = sc->interfaces;
+        if (!ifaces_array) continue;
+
+        while (*ifaces_array)
+          {
+             const Evas_Smart_Interface *iface = *ifaces_array;
+
+             if (!iface->name) break;
+
+             if (iface->private_size > 0)
+               {
+                  unsigned int size = iface->private_size;
+
+                  if (size % sizeof(void *) != 0)
+                    size += sizeof(void *) - (size % sizeof(void *));
+                  total_priv_sz += size;
+               }
+
+             ifaces_array++;
+          }
+     }
+
+   obj->interface_privates = malloc
+       (s->interfaces.size * sizeof(void *) + total_priv_sz);
+   if (!obj->interface_privates)
+     {
+        ERR("malloc failed!");
+        return;
+     }
+
+   /* make private data array ptrs point to right places, WHICH LIE ON
+    * THE SAME STRUCT, AFTER THE # OF INTERFACES COUNT */
+   ptr = (unsigned char *)(obj->interface_privates + s->interfaces.size);
+   for (i = 0; i < s->interfaces.size; i++)
+     {
+        unsigned int size;
+
+        size = s->interfaces.array[i]->private_size;
+
+        if (size == 0)
+          {
+             obj->interface_privates[i] = NULL;
+             continue;
+          }
+
+        obj->interface_privates[i] = ptr;
+        memset(ptr, 0, size);
+
+        if (size % sizeof(void *) != 0)
+          size += sizeof(void *) - (size % sizeof(void *));
+        ptr += size;
+     }
 }
 
 EAPI Evas_Object *
 evas_object_smart_add(Evas *e, Evas_Smart *s)
 {
    Evas_Object *obj;
+   unsigned int i;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return NULL;
@@ -320,6 +452,24 @@ evas_object_smart_add(Evas *e, Evas_Smart *s)
 
    evas_object_smart_use(s);
 
+   _evas_smart_class_ifaces_private_data_alloc(obj, s);
+
+   for (i = 0; i < s->interfaces.size; i++)
+     {
+        const Evas_Smart_Interface *iface;
+
+        iface = s->interfaces.array[i];
+        if (iface->add)
+          {
+             if (!iface->add(obj))
+               {
+                  ERR("failed to create interface %s\n", iface->name);
+                  evas_object_del(obj);
+                  return NULL;
+               }
+          }
+     }
+
    if (s->smart_class->add) s->smart_class->add(obj);
 
    return obj;
@@ -359,7 +509,7 @@ evas_object_smart_callback_priority_add(Evas_Object *obj, const char *event, Eva
    MAGIC_CHECK_END();
    if (!event) return;
    if (!func) return;
-   EVAS_MEMPOOL_INIT(_mp_cb, "evas_smart_callback", Evas_Smart_Callback, 512, );
+   EVAS_MEMPOOL_INIT(_mp_cb, "evas_smart_callback", Evas_Smart_Callback, 32, );
    cb = EVAS_MEMPOOL_ALLOC(_mp_cb, Evas_Smart_Callback);
    if (!cb) return;
    EVAS_MEMPOOL_PREP(_mp_cb, cb, Evas_Smart_Callback);
@@ -741,11 +891,25 @@ void
 evas_object_smart_del(Evas_Object *obj)
 {
    Evas_Smart *s;
+   unsigned int i;
 
    if (obj->delete_me) return;
    s = obj->smart.smart;
+
    if ((s) && (s->smart_class->del)) s->smart_class->del(obj);
    if (obj->smart.parent) evas_object_smart_member_del(obj);
+
+   for (i = 0; i < s->interfaces.size; i++)
+     {
+        const Evas_Smart_Interface *iface;
+
+        iface = s->interfaces.array[i];
+        if (iface->del) iface->del(obj);
+     }
+
+   free(obj->interface_privates);
+   obj->interface_privates = NULL;
+
    if (s) evas_object_smart_unuse(s);
 }
 
@@ -846,6 +1010,95 @@ evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other)
    o->contained = eina_inlist_prepend_relative(o->contained, EINA_INLIST_GET(member), EINA_INLIST_GET(other));
 }
 
+void
+evas_object_smart_need_bounding_box_update(Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(obj->object_data);
+
+   if (o->update_boundingbox_needed) return ;
+   o->update_boundingbox_needed = EINA_TRUE;
+
+   if (obj->smart.parent) evas_object_smart_need_bounding_box_update(obj->smart.parent);
+}
+
+void
+evas_object_smart_bouding_box_update(Evas_Object *obj)
+{
+   Eina_Inlist *list;
+   Evas_Object *o;
+   Evas_Object_Smart *os;
+   Evas_Coord minx;
+   Evas_Coord miny;
+   Evas_Coord maxw = 0;
+   Evas_Coord maxh = 0;
+
+   os = (Evas_Object_Smart *)(obj->object_data);
+
+   if (!os->update_boundingbox_needed) return ;
+   os->update_boundingbox_needed = EINA_FALSE;
+
+   minx = obj->layer->evas->output.w;
+   miny = obj->layer->evas->output.h;
+
+   list = os->contained;
+   EINA_INLIST_FOREACH(list, o)
+     {
+        Evas_Coord tx;
+        Evas_Coord ty;
+        Evas_Coord tw;
+        Evas_Coord th;
+
+        if (o == obj) continue ;
+        if (o->clip.clipees || o->is_static_clip) continue ;
+
+        if (o->smart.smart)
+          {
+             evas_object_smart_bouding_box_update(o);
+
+             tx = o->cur.bounding_box.x;
+             ty = o->cur.bounding_box.y;
+             tw = o->cur.bounding_box.x + o->cur.bounding_box.w;
+             th = o->cur.bounding_box.y + o->cur.bounding_box.h;
+          }
+        else
+          {
+             tx = o->cur.geometry.x;
+             ty = o->cur.geometry.y;
+             tw = o->cur.geometry.x + o->cur.geometry.w;
+             th = o->cur.geometry.y + o->cur.geometry.h;
+          }
+
+        if (tx < minx) minx = tx;
+        if (ty < miny) miny = ty;
+        if (tw > maxw) maxw = tw;
+        if (th > maxh) maxh = th;
+     }
+
+   if (minx != obj->cur.bounding_box.x)
+     {
+        obj->cur.bounding_box.w += obj->cur.bounding_box.x - minx;
+        obj->cur.bounding_box.x = minx;
+     }
+
+   if (miny != obj->cur.bounding_box.y)
+     {
+        obj->cur.bounding_box.h += obj->cur.bounding_box.y - miny;
+        obj->cur.bounding_box.y = miny;
+     }
+
+   if (maxw != obj->cur.bounding_box.x + obj->cur.bounding_box.w)
+     {
+        obj->cur.bounding_box.w = maxw - obj->cur.bounding_box.x;
+     }
+
+   if (maxh != obj->cur.bounding_box.y + obj->cur.bounding_box.h)
+     {
+        obj->cur.bounding_box.h = maxh - obj->cur.bounding_box.y;
+     }
+}
+
 /* all nice and private */
 static void
 evas_object_smart_init(Evas_Object *obj)
@@ -874,7 +1127,7 @@ evas_object_smart_new(void)
    Evas_Object_Smart *o;
 
    /* alloc obj private data */
-   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_smart", Evas_Object_Smart, 256, NULL);
+   EVAS_MEMPOOL_INIT(_mp_obj, "evas_object_smart", Evas_Object_Smart, 32, NULL);
    o = EVAS_MEMPOOL_ALLOC(_mp_obj, Evas_Object_Smart);
    if (!o) return NULL;
    EVAS_MEMPOOL_PREP(_mp_obj, o, Evas_Object_Smart);
@@ -906,21 +1159,95 @@ evas_object_smart_render(Evas_Object *obj __UNUSED__, void *output __UNUSED__, v
 static void
 evas_object_smart_render_pre(Evas_Object *obj)
 {
+   Evas *e;
+
    if (obj->pre_render_done) return;
-   if ((obj->cur.map != obj->prev.map) ||
-       (obj->cur.usemap != obj->prev.usemap))
+   if (!obj->child_has_map && !obj->cur.cached_surface)
      {
-        evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
-        goto done;
+#if 0
+        Evas_Object_Smart *o;
+
+        fprintf(stderr, "");
+        o = (Evas_Object_Smart *)(obj->object_data);
+        if (/* o->member_count > 1 && */
+            obj->cur.bounding_box.w == obj->prev.bounding_box.w &&
+            obj->cur.bounding_box.h == obj->prev.bounding_box.h &&
+            (obj->cur.bounding_box.x != obj->prev.bounding_box.x ||
+             obj->cur.bounding_box.y != obj->prev.bounding_box.y))
+          {
+             Eina_Bool cache_map = EINA_FALSE;
+
+             /* Check parent speed */
+             /* - same speed => do not map this object */
+             /* - different speed => map this object */
+             /* - if parent is mapped then map this object */
+
+             if (!obj->smart.parent || obj->smart.parent->child_has_map)
+               {
+                  cache_map = EINA_TRUE;
+               }
+             else
+               {
+                  if (_evas_render_has_map(obj->smart.parent))
+                    {
+                       cache_map = EINA_TRUE;
+                    }
+                  else
+                    {
+                       int speed_x, speed_y;
+                       int speed_px, speed_py;
+
+                       speed_x = obj->cur.geometry.x - obj->prev.geometry.x;
+                       speed_y = obj->cur.geometry.y - obj->prev.geometry.y;
+
+                       speed_px = obj->smart.parent->cur.geometry.x - obj->smart.parent->prev.geometry.x;
+                       speed_py = obj->smart.parent->cur.geometry.y - obj->smart.parent->prev.geometry.y;
+
+                       /* speed_x = obj->cur.bounding_box.x - obj->prev.bounding_box.x; */
+                       /* speed_y = obj->cur.bounding_box.y - obj->prev.bounding_box.y; */
+
+                       /* speed_px = obj->smart.parent->cur.bounding_box.x - obj->smart.parent->prev.bounding_box.x; */
+                       /* speed_py = obj->smart.parent->cur.bounding_box.y - obj->smart.parent->prev.bounding_box.y; */
+
+                       fprintf(stderr, "speed: '%s',%p (%i, %i) vs '%s',%p (%i, %i)\n",
+                               evas_object_type_get(obj), obj, speed_x, speed_y,
+                               evas_object_type_get(obj->smart.parent), obj->smart.parent, speed_px, speed_py);
+
+                       if (speed_x != speed_px || speed_y != speed_py)
+                         cache_map = EINA_TRUE;
+                    }
+               }
+
+             if (cache_map)
+               fprintf(stderr, "Wouhou, I can detect moving smart object (%s, %p [%i, %i, %i, %i] < %s, %p [%i, %i, %i, %i])\n",
+                       evas_object_type_get(obj), obj,
+                       obj->cur.bounding_box.x - obj->prev.bounding_box.x,
+                       obj->cur.bounding_box.y - obj->prev.bounding_box.y,
+                       obj->cur.bounding_box.w, obj->cur.bounding_box.h,
+                       evas_object_type_get(obj->smart.parent), obj->smart.parent,
+                       obj->smart.parent->cur.bounding_box.x - obj->smart.parent->prev.bounding_box.x,
+                       obj->smart.parent->cur.bounding_box.y - obj->smart.parent->prev.bounding_box.y,
+                       obj->smart.parent->cur.bounding_box.w, obj->smart.parent->cur.bounding_box.h);
+
+             obj->cur.cached_surface = cache_map;
+          }
+#endif
      }
-done:
-   obj->pre_render_done = 1;
+
+   e = obj->layer->evas;
+
+   if (obj->changed_map)
+     {
+       evas_object_render_pre_prev_cur_add(&e->clip_changes, obj);
+     }
+
+   obj->pre_render_done = EINA_TRUE;
 }
 
 static void
 evas_object_smart_render_post(Evas_Object *obj)
 {
-   obj->prev = obj->cur;
+   evas_object_cur_prev(obj);
 }
 
 static unsigned int evas_object_smart_id_get(Evas_Object *obj)
