@@ -363,14 +363,6 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
 
    o->font = evas_font_load(obj->layer->evas, o->cur.fdesc, o->cur.source,
          (int)(((double) o->cur.size) * obj->cur.scale));
-   if (o->font)
-     {
-        o->ascent = ENFN->font_ascent_get(ENDT, o->font);
-        o->descent = ENFN->font_descent_get(ENDT, o->font);
-        o->max_ascent = ENFN->font_max_ascent_get(ENDT, o->font);
-        o->max_descent = ENFN->font_max_descent_get(ENDT, o->font);
-     }
-   else
      {
         o->ascent = 0;
         o->descent = 0;
@@ -1917,6 +1909,7 @@ _evas_object_text_recalc(Evas_Object *obj)
    if (text) free(text);
 
    /* Calc ascent/descent. */
+   if (o->items)
      {
         Evas_Object_Text_Item *item;
 
@@ -1925,28 +1918,28 @@ _evas_object_text_recalc(Evas_Object *obj)
                  EINA_INLIST_GET(item)->next, Evas_Object_Text_Item))
           {
              int asc = 0, desc = 0;
-             int max_asc = 0, max_desc = 0;
 
              /* Skip items without meaning full information. */
-             if (!item->text_props.font_instance)
-                continue;
+             if (!item->text_props.font_instance) continue;
 
              asc = evas_common_font_instance_ascent_get(item->text_props.font_instance);
              desc = evas_common_font_instance_descent_get(item->text_props.font_instance);
-             if (asc > o->ascent)
-                o->ascent = asc;
-             if (desc > o->descent)
-                o->descent = desc;
+             if (asc > o->ascent) o->ascent = asc;
+             if (desc > o->descent) o->descent = desc;
 
-             max_asc = evas_common_font_instance_max_ascent_get(item->text_props.font_instance);
-             max_desc = evas_common_font_instance_max_descent_get(item->text_props.font_instance);
-
-             if (max_asc > o->max_ascent)
-                o->max_ascent = max_asc;
-             if (max_desc > o->max_descent)
-                o->max_descent = max_desc;
+             asc = evas_common_font_instance_max_ascent_get(item->text_props.font_instance);
+             desc = evas_common_font_instance_max_descent_get(item->text_props.font_instance);
+             if (asc > o->max_ascent) o->max_ascent = asc;
+             if (desc > o->max_descent) o->max_descent = desc;
           }
      }
+   else if (o->font)
+     {
+        o->ascent = ENFN->font_ascent_get(ENDT, o->font);
+        o->descent = ENFN->font_descent_get(ENDT, o->font);
+        o->max_ascent = ENFN->font_max_ascent_get(ENDT, o->font);
+        o->max_descent = ENFN->font_max_descent_get(ENDT, o->font);
+     }
 
    if ((o->font) && (o->items))
      {
