@@ -1,15 +1,12 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #include "evas_common.h"
 #include "evas_convert_color.h"
 
 
-EAPI void
+EAPI DATA32
 evas_common_convert_argb_premul(DATA32 *data, unsigned int len)
 {
-   DATA32  *de = data + len;
+   DATA32 *de = data + len;
+   DATA32 nas = 0;
 
    while (data < de)
      {
@@ -19,7 +16,12 @@ evas_common_convert_argb_premul(DATA32 *data, unsigned int len)
 	  (((((*data) >> 8) & 0xff) * a) & 0xff00) +
 	  (((((*data) & 0x00ff00ff) * a) >> 8) & 0x00ff00ff);
 	data++;
+
+	if ((a == 1) || (a == 256))
+	  nas++;
      }
+
+   return nas;
 }
 
 EAPI void
