@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #include "evas_common.h"
 #include "evas_convert_yuv.h"
 
@@ -29,6 +25,9 @@ static void _evas_yv12torgb_altivec(unsigned char **yuv, unsigned char *rgb, int
 static void _evas_yv12torgb_diz    (unsigned char **yuv, unsigned char *rgb, int w, int h);
 #endif
 static void _evas_yv12torgb_raster (unsigned char **yuv, unsigned char *rgb, int w, int h);
+static void _evas_yuy2torgb_raster (unsigned char **yuv, unsigned char *rgb, int w, int h);
+static void _evas_nv12torgb_raster (unsigned char **yuv, unsigned char *rgb, int w, int h);
+static void _evas_nv12tiledtorgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h);
 
 #define CRV    104595
 #define CBU    132251
@@ -123,6 +122,8 @@ static unsigned char _clip_lut[1024];
 
 #define CMP_CLIP(i) ((i&256)? (~(i>>10)) : i);
 
+static int initted = 0;
+
 #endif
 
 void
@@ -151,8 +152,6 @@ evas_common_convert_yuv_420p_601_rgba(DATA8 **src, DATA8 *dst, int w, int h)
    else
      {
 #ifdef BUILD_C
-	static int initted = 0;
-
 	if (!initted) _evas_yuv_init();
 	initted = 1;
 	/* FIXME: diz may be faster sometimes */
@@ -164,7 +163,7 @@ evas_common_convert_yuv_420p_601_rgba(DATA8 **src, DATA8 *dst, int w, int h)
 /* Thanks to Diz for this code. i've munged it a little and turned it into */
 /* inline macros. I tried beating it with a different algorithm using MMX */
 /* but failed. So here we are. This is the fastest YUV->RGB i know of for */
-/* x86. It has an issue that it doesnt convert colours accurately so the */
+/* x86. It has an issue that it doesn't convert colours accurately so the */
 /* image looks a little "yellowy". This is a result of only 10.6 fixed point */
 /* resolution as opposed to 16.16 in the C code. This could be fixed by */
 /* processing half the number of pixels per cycle and going up to 32bits */
@@ -561,7 +560,7 @@ _evas_yv12torgb_altivec(unsigned char **yuv, unsigned char *rgb, int w, int h)
 /* 2 */      tmp1 = vec_perm(tmp1, tmp1, uperm);
 /* 2 */      tmp2 = vec_perm(tmp2, tmp2, vperm);
 
-	     /* Avoid dependancy stalls on yperm and calculate the 4 u values */
+	     /* Avoid dependency stalls on yperm and calculate the 4 u values */
 /* 3 */      yperm = vec_lvsr(12, yp1);
 /* 1 */      tmp1 = (vector signed short)vec_mergeh((vector unsigned char)tmp1,
 						    (vector unsigned char)tmp1);
@@ -590,7 +589,7 @@ _evas_yv12torgb_altivec(unsigned char **yuv, unsigned char *rgb, int w, int h)
 /* 1 */      y = vec_sub(y, c16);
 /* 4 */      y = vec_mladd(ymul, y, (vector signed short)zero);
 
-	     /* Perform non-dependant multiplies first. */
+	     /* Perform non-dependent multiplies first. */
 /* 4 */      tmp1 = vec_mladd(crv, v, y);
 /* 4 */      tmp2 = vec_mladd(cgv, v, tmp2);
 /* 4 */      tmp3 = vec_mladd(cbu, u, y);
@@ -647,7 +646,7 @@ _evas_yv12torgb_altivec(unsigned char **yuv, unsigned char *rgb, int w, int h)
 /* 1 */      y = (vector signed short)vec_mergeh(zero,
 						 (vector unsigned char)y);
 
-	     /* Avoid dependancy stalls on yperm */
+	     /* Avoid dependency stalls on yperm */
 /* 2 */      tmp3 = vec_perm(tmp3, tmp3, yperm);
 /* 1 */      tmp3 = (vector signed short)vec_mergeh(zero,
 						    (vector unsigned char)tmp3);
@@ -660,7 +659,7 @@ _evas_yv12torgb_altivec(unsigned char **yuv, unsigned char *rgb, int w, int h)
 /* 1 */      y = vec_sub(y, c16);
 /* 4 */      y = vec_mladd(ymul, y, (vector signed short)zero);
 
-	     /* Perform non-dependant multiplies first. */
+	     /* Perform non-dependent multiplies first. */
 /* 4 */      tmp2 = vec_mladd(cgv, v, tmp2);
 /* 4 */      tmp1 = vec_mladd(crv, v, y);
 /* 4 */      tmp3 = vec_mladd(cbu, u, y);
@@ -779,7 +778,7 @@ _evas_yv12torgb_diz(unsigned char **yuv, unsigned char *rgb, int w, int h)
 	     u = (*up++) - 128;
 	     v = (*vp++) - 128;
 
-	     /* do the top 2 pixels of the 2x2 block whcih shared u & v */
+	     /* do the top 2 pixels of the 2x2 block which shared u & v */
 	     /* yuv to rgb */
 	     y = YMUL * ((*yp1++) - 16);
 	     r = LUT_CLIP((y + (crv * v)) >> 16);
@@ -858,7 +857,7 @@ _evas_yv12torgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h)
 	     u = _v2018[u];
 	     v = _v1596[v];
 
-             /* do the top 2 pixels of the 2x2 block whcih shared u & v */
+             /* do the top 2 pixels of the 2x2 block which shared u & v */
 	     /* yuv to rgb */
 	     y = _v1164[*yp1++];
 	     *((DATA32 *) dp1) = 0xff000000 + RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
@@ -881,7 +880,7 @@ _evas_yv12torgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h)
 	     /* yuv to rgb */
 	     y = _v1164[*yp2++];
 	     *((DATA32 *) dp2) = 0xff000000 + RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
-	     
+
 	     dp2 += 4;
 	  }
 	/* jump down one line since we are doing 2 at once */
@@ -891,5 +890,369 @@ _evas_yv12torgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h)
 #endif
 }
 
+void
+evas_common_convert_yuv_422_601_rgba(DATA8 **src, DATA8 *dst, int w, int h)
+{
+#ifdef BUILD_C
+   if (!initted) _evas_yuv_init();
+   initted = 1;
+   _evas_yuy2torgb_raster(src, dst, w, h);
+#endif
+}
+
+void
+evas_common_convert_yuv_420_601_rgba(DATA8 **src, DATA8 *dst, int w, int h)
+{
+#ifdef BUILD_C
+   if (!initted) _evas_yuv_init();
+   initted = 1;
+   _evas_nv12torgb_raster(src, dst, w, h);
+#endif
+}
+
+void
+evas_common_convert_yuv_420T_601_rgba(DATA8 **src, DATA8 *dst, int w, int h)
+{
+#ifdef BUILD_C
+   if (initted) _evas_yuv_init();
+   initted = 1;
+   _evas_nv12tiledtorgb_raster(src, dst, w, h);
+#endif
+}
+
+static void
+_evas_yuy2torgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h)
+{
+#ifdef BUILD_C
+   int xx, yy;
+   int y, u, v;
+   unsigned char *yp1, *yp2, *up, *vp;
+   unsigned char *dp1;
+
+   dp1 = rgb;
+
+   /* destination pointers */
+   for (yy = 0; yy < h; yy++)
+     {
+        /* plane pointers */
+        unsigned char *line;
+
+        line = yuv[yy];
+        yp1 = line + 0;
+        up = line + 1;
+        yp2 = line + 2;
+        vp = line + 3;
+
+        for (xx = 0; xx < w; xx += 2)
+          {
+             int vmu;
+
+             /* collect u & v for 2 pixels block */
+             u = *up;
+             v = *vp;
+
+             /* save lookups */
+             vmu = _v813[v] + _v391[u];
+             u = _v2018[u];
+             v = _v1596[v];
+
+             /* do the top 2 pixels of the 2x2 block which shared u & v */
+	     /* yuv to rgb */
+	     y = _v1164[*yp1];
+	     *((DATA32 *) dp1) = 0xff000000 + RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
+
+	     dp1 += 4;
+
+	     /* yuv to rgb */
+	     y = _v1164[*yp2];
+	     *((DATA32 *) dp1) = 0xff000000 + RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
+
+             dp1 += 4;
+
+	     yp1 += 4; yp2 += 4; up += 4; vp += 4;
+	  }
+     }
+#endif
+}
+
+#ifdef BUILD_C
+static inline void
+_evas_yuv2rgb_420_raster(unsigned char *yp1, unsigned char *yp2, unsigned char *up, unsigned char *vp,
+                         unsigned char *dp1, unsigned char *dp2)
+{
+   int y, u, v;
+   int vmu;
+   int rgb;
+
+   /* collect u & v for 4 pixels block */
+   u = *up;
+   v = *vp;
+
+   /* save lookups */
+#ifdef MEM_BP
+   vmu = _v813[v] + _v391[u];
+   u = _v2018[u];
+   v = _v1596[v];
+#else
+   u -= 128;
+   v -= 128;
+   vmu = v * CGV + u * CGU;
+   u = u * CBU;
+   v = v * CRV;
+#endif
+
+   /* do the top 2 pixels of the 2x2 block which shared u & v */
+   /* yuv to rgb */
+#ifdef MEM_BP
+   y = _v1164[*yp1];
+   rgb = RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
+#else
+   y = (*yp1 - 16 ) * YMUL;
+   rgb = RGB_JOIN(LUT_CLIP(((y + v) >> 16)),
+                  LUT_CLIP(((y - vmu + OFF) >> 16)),
+                  LUT_CLIP(((y + u + OFF) >> 16)));
+#endif
+   *((DATA32 *) dp1) = 0xff000000 + rgb;
+
+   dp1 += 4; yp1++;
+
+   /* yuv to rgb */
+#ifdef MEM_BP
+   y = _v1164[*yp1];
+   rgb = RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
+#else
+   y = (*yp1 - 16 ) * YMUL;
+   rgb = RGB_JOIN(LUT_CLIP(((y + v) >> 16)),
+                  LUT_CLIP(((y - vmu + OFF) >> 16)),
+                  LUT_CLIP(((y + u + OFF) >> 16)));
+#endif
+   *((DATA32 *) dp1) = 0xff000000 + rgb;
+
+   /* do the bottom 2 pixels of the 2x2 block which shared u & v */
+   /* yuv to rgb */
+#ifdef MEM_BP
+   y = _v1164[*yp2];
+   rgb = RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
+#else
+   y = (*yp2 - 16 ) * YMUL;
+   rgb = RGB_JOIN(LUT_CLIP(((y + v) >> 16)),
+                  LUT_CLIP(((y - vmu + OFF) >> 16)),
+                  LUT_CLIP(((y + u + OFF) >> 16)));
+#endif
+   *((DATA32 *) dp2) = 0xff000000 + rgb;
+
+   dp2 += 4; yp2++;
+
+   /* yuv to rgb */
+#ifdef MEM_BP
+   y = _v1164[*yp2];
+   rgb = RGB_JOIN(LUT_CLIP(y + v), LUT_CLIP(y - vmu), LUT_CLIP(y + u));
+#else
+   y = (*yp2 - 16 ) * YMUL;
+   rgb = RGB_JOIN(LUT_CLIP(((y + v) >> 16)),
+                  LUT_CLIP(((y - vmu + OFF) >> 16)),
+                  LUT_CLIP(((y + u + OFF) >> 16)));
+#endif
+   *((DATA32 *) dp2) = 0xff000000 + rgb;
+}
+#endif
+
+static void
+_evas_nv12tiledtorgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h)
+{
+#ifdef BUILD_C
+
+#define HANDLE_MACROBLOCK(YP1, YP2, UP, VP, DP1, DP2)                   \
+   {                                                                    \
+     int i;                                                             \
+     int j;                                                             \
+                                                                        \
+     for (i = 0; i < 32; i += 2)                                        \
+       {                                                                \
+          for (j = 0; j < 64; j += 2)                                   \
+            {                                                           \
+               _evas_yuv2rgb_420_raster(YP1, YP2, UP, VP, DP1, DP2);    \
+                                                                        \
+               /* the previous call just rendered 2 pixels per lines */ \
+               DP1 += 8; DP2 += 8;                                      \
+                                                                        \
+               /* and took for that 2 lines with 2 Y, 1 U and 1 V. Don't forget U & V are in the same plane */ \
+               YP1 += 2; YP2 += 2; UP += 2; VP += 2;                    \
+            }                                                           \
+                                                                        \
+          DP1 += sizeof (int) * ((w << 1) - 64);			\
+          DP2 += sizeof (int) * ((w << 1) - 64);			\
+          YP1 += 64;                                                    \
+          YP2 += 64;                                                    \
+       }                                                                \
+   }
+
+   /* One macro block is 32 lines of Y and 16 lines of UV */
+   const int offset_value[2] = { 0, 64 * 16 };
+   int mb_x, mb_y, mb_w, mb_h;
+   int base_h;
+   int uv_x, uv_step;
+   int stride;
+
+   /* Idea iterate over each macroblock and convert each of them using _evas_nv12torgb_raster */
+
+   /* The layout of the Y macroblock order in RGB non tiled space : */
+   /* --------------------------------------------------- */
+   /* | 0  | 1  | 6  | 7  | 8  | 9  | 14 | 15 | 16 | 17 | */
+   /* --------------------------------------------------- */
+   /* | 2  | 3  | 4  | 5  | 10 | 11 | 12 | 13 | 18 | 19 | */
+   /* --------------------------------------------------- */
+   /* | 20 | 21 | 26 | 27 | 28 | 29 | 34 | 35 | 36 | 37 | */
+   /* --------------------------------------------------- */
+   /* | 22 | 23 | 24 | 25 | 30 | 31 | 32 | 33 | 38 | 39 | */
+   /* --------------------------------------------------- */
+   /* | 40 | 41 | 42 | 43 | 44 | 45 | 46 | 47 | 48 | 49 | */
+   /* --------------------------------------------------- */
+   /* The layout of the UV macroblock order in the same RGB non tiled space : */
+   /* --------------------------------------------------- */
+   /* |    |    |    |    |    |    |    |    |    |    | */
+   /* - 0  - 1  - 6  - 7  - 8  - 9  - 14 - 15 - 16 - 17 - */
+   /* |    |    |    |    |    |    |    |    |    |    | */
+   /* --------------------------------------------------- */
+   /* |    |    |    |    |    |    |    |    |    |    | */
+   /* - 2  - 3  - 4  - 5  - 10 - 11 - 12 - 13 - 18 - 19 - */
+   /* |    |    |    |    |    |    |    |    |    |    | */
+   /* --------------------------------------------------- */
+   /* |    |    |    |    |    |    |    |    |    |    | */
+   /* - 20 - 21 - 22 - 22 - 23 - 24 - 25 - 26 - 27 - 28 - */
+
+   /* the number of macroblock should be a multiple of 64x32 */
+   mb_w = w / 64;
+   mb_h = h / 32;
+
+   base_h = (mb_h >> 1) + (mb_h & 0x1);
+   stride = w * sizeof (int);
+
+   uv_x = 0;
+
+   /* In this format we linearize macroblock on two line to form a Z and it's invert */
+   for (mb_y = 0; mb_y < (mb_h >> 1); mb_y++)
+     {
+        int step = 2;
+        int offset = 0;
+        int x = 0;
+	int rmb_x = 0;
+	int ry[2];
+
+	ry[0] = mb_y * 2 * 32 * stride;
+	ry[1] = ry[0] + 32 * stride;
+
+	uv_step = (mb_y & 0x1) == 0 ? 4 : 0;
+	uv_x = (mb_y & 0x1) == 0 ? 0 : 2 * 64 * 32;
+
+	for (mb_x = 0; mb_x < mb_w * 2; mb_x++, rmb_x += 64 * 32)
+	  {
+	    unsigned char *yp1, *yp2, *up, *vp;
+	    unsigned char *dp1, *dp2;
+
+	    dp1 = rgb + x + ry[offset];
+	    dp2 = dp1 + stride;
+
+	    yp1 = yuv[mb_y] + rmb_x;
+	    yp2 = yp1 + 64;
+
+	    /* UV plane is two time less bigger in pixel count, but it old two bytes each times */
+	    up = yuv[(mb_y >> 1) + base_h] + uv_x + offset_value[offset];
+	    vp = up + 1;
+
+	    HANDLE_MACROBLOCK(yp1, yp2, up, vp, dp1, dp2);
+
+	    step++;
+	    if ((step & 0x3) == 0)
+	      {
+		offset = 1 - offset;
+		x -= 64 * sizeof (int);
+		uv_x -= 64 * 32;
+	      }
+	    else
+	      {
+		x += 64 * sizeof (int);
+		uv_x += 64 * 32;
+	      }
+
+	    uv_step++;
+	    if (uv_step == 8)
+	      {
+		uv_step = 0;
+		uv_x += 4 * 64 * 32;
+	      }
+	  }
+     }
+
+   if (mb_h & 0x1)
+     {
+        int x = 0;
+	int ry;
+
+	ry = mb_y << 1;
+
+	uv_step = 0;
+	uv_x = 0;
+
+        for (mb_x = 0; mb_x < mb_w; mb_x++, x++, uv_x++)
+          {
+             unsigned char *yp1, *yp2, *up, *vp;
+             unsigned char *dp1, *dp2;
+
+             dp1 = rgb + (x * 64 + (ry * 32 * w)) * sizeof (int);
+             dp2 = dp1 + sizeof (int) * w;
+
+             yp1 = yuv[mb_y] + mb_x * 64 * 32;
+             yp2 = yp1 + 64;
+
+             up = yuv[mb_y / 2 + base_h] + uv_x * 64 * 32;
+             vp = up + 1;
+
+             HANDLE_MACROBLOCK(yp1, yp2, up, vp, dp1, dp2);
+          }
+     }
+#endif
+}
+
+static void
+_evas_nv12torgb_raster(unsigned char **yuv, unsigned char *rgb, int w, int h)
+{
+#ifdef BUILD_C
+   int xx, yy;
+   unsigned char *yp1, *yp2, *up, *vp;
+   unsigned char *dp1;
+   unsigned char *dp2;
+
+   dp1 = rgb;
+   dp2 = dp1 + sizeof (int) * w;
+
+   for (yy = 0; yy < h; yy++)
+     {
+        yp1 = yuv[yy++];
+        yp2 = yuv[yy];
+
+        up = yuv[h + (yy >> 1)];
+        vp = up + 1;
+
+        for (xx = 0; xx < w; xx += 2)
+          {
+             _evas_yuv2rgb_420_raster(yp1, yp2, up, vp, dp1, dp2);
+
+             /* the previous call just rendered 2 pixels per lines */
+             dp1 += 8; dp2 += 8;
+
+             /* and took for that 2 lines with 2 Y, 1 U and 1 V. Don't forget U & V are in the same plane */
+             yp1 += 2; yp2 += 2; up += 2; vp += 2;
+          }
+
+        /* jump one line */
+        dp1 += sizeof (int) * w;
+        dp2 += sizeof (int) * w;
+        yp1 += w;
+        yp2 += w;
+     }
+#endif
+}
+
 #endif
 
