@@ -3,7 +3,11 @@
 #include "evas_mmx.h"
 #endif
 
-#ifndef _WIN32
+#if defined BUILD_SSE3
+#include <immintrin.h>
+#endif
+
+#if defined (HAVE_STRUCT_SIGACTION) && defined (HAVE_SIGLONGJMP)
 #include <signal.h>
 #include <setjmp.h>
 #include <errno.h>
@@ -13,7 +17,7 @@ static sigjmp_buf detect_buf;
 
 static int cpu_feature_mask = 0;
 
-#ifndef _WIN32
+#if defined (HAVE_STRUCT_SIGACTION) && defined (HAVE_SIGLONGJMP)
 static void evas_common_cpu_catch_ill(int sig);
 static void evas_common_cpu_catch_segv(int sig);
 
@@ -60,6 +64,17 @@ evas_common_cpu_sse_test(void)
 #endif
 }
 
+void evas_common_op_sse3_test(void);
+
+void
+evas_common_cpu_sse3_test(void)
+{
+#ifdef BUILD_SSE3
+   evas_common_op_sse3_test();
+#endif
+}
+
+#ifdef BUILD_ALTIVEC
 void
 evas_common_cpu_altivec_test(void)
 {
@@ -71,6 +86,7 @@ evas_common_cpu_altivec_test(void)
 #endif /* __VEC__ */
 #endif /* __POWERPC__ */
 }
+#endif /* BUILD_ALTIVEC */
 
 void
 evas_common_cpu_neon_test(void)
@@ -99,7 +115,7 @@ evas_common_cpu_vis_test(void)
 int
 evas_common_cpu_feature_test(void (*feature)(void))
 {
-#ifndef _WIN32
+#if defined (HAVE_STRUCT_SIGACTION) && defined (HAVE_SIGLONGJMP)
    int enabled = 1;
    struct sigaction act, oact, oact2;
 
@@ -154,8 +170,16 @@ evas_common_cpu_init(void)
    evas_common_cpu_end_opt();
    if (getenv("EVAS_CPU_NO_SSE"))
      cpu_feature_mask &= ~CPU_FEATURE_SSE;
+#ifdef BUILD_SSE3
+   cpu_feature_mask |= CPU_FEATURE_SSE3 *
+     evas_common_cpu_feature_test(evas_common_cpu_sse3_test); 
+   evas_common_cpu_end_opt();
+   if (getenv("EVAS_CPU_NO_SSE3"))
+     cpu_feature_mask &= ~CPU_FEATURE_SSE3; 
+#endif /* BUILD_SSE3 */
 #endif /* BUILD_SSE */
 #endif /* BUILD_MMX */
+#ifdef BUILD_ALTIVEC
 #ifdef __POWERPC__
 #ifdef __VEC__
    cpu_feature_mask |= CPU_FEATURE_ALTIVEC *
@@ -165,6 +189,7 @@ evas_common_cpu_init(void)
      cpu_feature_mask &= ~CPU_FEATURE_ALTIVEC;
 #endif /* __VEC__ */
 #endif /* __POWERPC__ */
+#endif /* BUILD_ALTIVEC */
 #ifdef __SPARC__
    cpu_feature_mask |= CPU_FEATURE_VIS *
      evas_common_cpu_feature_test(evas_common_cpu_vis_test);
