@@ -79,12 +79,10 @@ evas_common_font_query_run_font_end_get(RGBA_Font *fn, RGBA_Font_Int **script_fi
                 continue;
              /* Break if either it's not in the font, or if it is in the
               * script's font. */
-             if (fi == *script_fi)
-               {
-                  if (!evas_common_get_char_index(fi, *itr))
-                     break;
-               }
-             else
+             if (!evas_common_get_char_index(fi, *itr))
+                break;
+
+             if (fi != *script_fi)
                {
                   if (evas_common_get_char_index(*script_fi, *itr))
                      break;
@@ -274,6 +272,46 @@ evas_common_font_query_right_inset(RGBA_Font *fn __UNUSED__, const Evas_Text_Pro
       );
 }
 
+/**
+ * @internal
+ * Calculate the ascent/descent of a run. This is different from
+ * evas_common_font_[max]_ascent/descent_get because this one returns the
+ * actual sizee (i.e including accents), and not just what the font reports.
+ *
+ * @param fn the font set to use.
+ * @param text_props the string object.
+ * @param[out] ascent the calculated ascent
+ * @param[out] descent the calculated descent
+ */
+EAPI void
+evas_common_font_ascent_descent_get(RGBA_Font *fn, const Evas_Text_Props *text_props, int *ascent, int *descent)
+{
+   int asc = 0, desc = 0;
+   int max_asc, max_desc;
+
+   max_asc = evas_common_font_ascent_get(fn);
+   max_desc = evas_common_font_descent_get(fn);
+
+   /* FIXME: Not supported yet!!! */
+   desc = max_desc;
+
+   EVAS_FONT_WALK_TEXT_INIT();
+   EVAS_FONT_WALK_TEXT_START()
+     {
+        EVAS_FONT_WALK_TEXT_WORK();
+        if (!EVAS_FONT_WALK_IS_VISIBLE) continue;
+
+        if ((EVAS_FONT_WALK_PEN_Y + EVAS_FONT_WALK_Y_OFF + EVAS_FONT_WALK_Y_BEAR) > asc)
+          {
+             asc = EVAS_FONT_WALK_PEN_Y + EVAS_FONT_WALK_Y_OFF + EVAS_FONT_WALK_Y_BEAR;
+          }
+     }
+   EVAS_FONT_WALK_TEXT_END();
+
+   if (ascent) *ascent = (asc > max_asc) ? asc : max_asc;
+   if (descent) *descent = (desc < max_desc) ? desc : max_desc;
+}
+
 /**
  * @internal
  * Calculate the size of the string (width and height).
@@ -331,11 +369,24 @@ evas_common_font_query_advance(RGBA_Font *fn, const Evas_Text_Props *text_props,
    Evas_Coord ret_adv = 0;
    if (text_props->len > 0)
      {
+//        RGBA_Font_Int *fi = text_props->font_instance;
         const Evas_Font_Glyph_Info *glyph = text_props->info->glyph +
            text_props->start;
-        ret_adv = glyph[text_props->len - 1].pen_after;
+        const Evas_Font_Glyph_Info *last_glyph = glyph + text_props->len - 1;
+        ret_adv = last_glyph->pen_after;
         if (text_props->start > 0)
            ret_adv -= glyph[-1].pen_after;
+
+#if 0
+        /* Runtime slant adjustment. */
+        if (fi->runtime_rend & FONT_REND_SLANT)
+          {
+             RGBA_Font_Glyph *fg =
+                evas_common_font_int_cache_glyph_get(fi, last_glyph->index);
+             if (!fg->glyph_out) evas_common_font_int_cache_glyph_render(fg);
+             ret_adv += fg->glyph_out->bitmap.rows * _EVAS_FONT_SLANT_TAN;
+          }
+#endif
      }
 
    if (h_adv) *h_adv = ret_adv;
