@@ -19,6 +19,9 @@
 //#define SURFDBG 1
 
 static Evas_Cache_Image * eci = NULL;
+#ifdef EVAS_CSERVE2
+static Evas_Cache2      * eci2 = NULL;
+#endif
 static int                reference = 0;
 
 /* static RGBA_Image *evas_rgba_line_buffer = NULL; */
@@ -78,11 +81,38 @@ static const Evas_Cache_Image_Func      _evas_common_image_func =
   NULL
 };
 
+#ifdef EVAS_CSERVE2
+static const Evas_Cache2_Image_Func      _evas_common_image_func2 =
+{
+  // _evas_common_rgba_image_new,
+  // _evas_common_rgba_image_delete,
+  _evas_common_rgba_image_surface_alloc,
+  _evas_common_rgba_image_surface_delete,
+  _evas_common_rgba_image_surface_pixels,
+  // evas_common_load_rgba_image_module_from_file,
+  // _evas_common_rgba_image_unload,
+  NULL, // _evas_common_rgba_image_dirty_region,
+  NULL, // _evas_common_rgba_image_dirty,
+  evas_common_rgba_image_size_set,
+  evas_common_rgba_image_from_copied_data,
+  evas_common_rgba_image_from_data,
+  NULL, // evas_common_rgba_image_colorspace_set,
+  // evas_common_load_rgba_image_data_from_file,
+  _evas_common_rgba_image_ram_usage,
+/*   _evas_common_rgba_image_debug */
+  NULL
+};
+#endif
+
 EAPI void
 evas_common_image_init(void)
 {
    if (!eci)
      eci = evas_cache_image_init(&_evas_common_image_func);
+#ifdef EVAS_CSERVE2
+   if (!eci2)
+     eci2 = evas_cache2_init(&_evas_common_image_func2);
+#endif
    reference++;
 ////   ERR("REF++=%i", reference);
 
@@ -115,6 +145,10 @@ evas_common_image_shutdown(void)
 // ENABLE IT AGAIN, hope it is fixed. Gustavo @ January 22nd, 2009.
        evas_cache_image_shutdown(eci);
        eci = NULL;
+#ifdef EVAS_CSERVE2
+       evas_cache2_shutdown(eci2);
+       eci2 = NULL;
+#endif
      }
 
 #ifdef BUILD_LOADER_EET
@@ -139,12 +173,6 @@ _evas_common_rgba_image_new(void)
    if (!im) return NULL;
    im->flags = RGBA_IMAGE_NOTHING;
    im->ref = 1;
-#ifdef EVAS_FRAME_QUEUING
-   LKI(im->cache_entry.ref_fq_add);
-   LKI(im->cache_entry.ref_fq_del);
-   eina_condition_new(&(im->cache_entry.cond_fq_del),
-		      &(im->cache_entry.ref_fq_del));
-#endif
 
    evas_common_rgba_image_scalecache_init(&im->cache_entry);
 
@@ -158,11 +186,6 @@ _evas_common_rgba_image_delete(Image_Entry *ie)
 
 #ifdef BUILD_PIPE_RENDER
    evas_common_pipe_free(im);
-# ifdef EVAS_FRAME_QUEUING
-   LKD(im->cache_entry.ref_fq_add);
-   LKD(im->cache_entry.ref_fq_del);
-   eina_condition_free(&(im->cache_entry.cond_fq_del));
-# endif
 #endif
    evas_common_rgba_image_scalecache_shutdown(&im->cache_entry);
    if (ie->info.module) evas_module_unref((Evas_Module *)ie->info.module);
@@ -176,6 +199,12 @@ _evas_common_rgba_image_delete(Image_Entry *ie)
 #ifdef EVAS_CSERVE
    if (ie->data1) evas_cserve_image_free(ie);
 #endif
+#ifdef EVAS_CSERVE2
+   if (ie->data1)
+       ERR("Shouldn't reach this point since we are using cache2: '%s' '%s'",
+           ie->file, ie->key);
+   // if (ie->data1) evas_cserve2_image_free(ie);
+#endif
 /*
  * FIXME: This doesn't seem to be needed... But I'm not sure why.
  *	 -- nash
@@ -277,6 +306,22 @@ evas_common_rgba_image_unload(Image_Entry *ie)
      }
 #endif
 
+#ifdef EVAS_CSERVE2
+   if (ie->data1)
+     {
+        ERR("Shouldn't reach this point since we are using cache2.");
+//         evas_cserve2_image_unload(ie);
+//         im->image.data = NULL;
+//         ie->allocated.w = 0;
+//         ie->allocated.h = 0;
+//         ie->flags.loaded = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+        ie->flags.preload_done = 0;
+#endif
+        return;
+     }
+#endif
+
    if (im->image.data && !im->image.no_free)
      {
         free(im->image.data);
@@ -302,8 +347,16 @@ _evas_common_rgba_image_post_surface(Image_Entry *ie)
 #ifdef HAVE_PIXMAN
 # ifdef PIXMAN_IMAGE   
    RGBA_Image *im = (RGBA_Image *)ie;
-
+   int w, h;
+   
    if (im->pixman.im) pixman_image_unref(im->pixman.im);
+   w = ie->allocated.w;
+   h = ie->allocated.h;
+   if ((w <= 0) || (h <= 0))
+     {
+        w = im->cache_entry.w;
+        h = im->cache_entry.h;
+     }
    if (im->cache_entry.flags.alpha)
      {
         im->pixman.im = pixman_image_create_bits
@@ -311,10 +364,7 @@ _evas_common_rgba_image_post_surface(Image_Entry *ie)
 // FIXME: endianess determines this
             PIXMAN_a8r8g8b8,
 //            PIXMAN_b8g8r8a8,
-            im->cache_entry.w, im->cache_entry.h,
-            im->image.data,
-            im->cache_entry.w * 4
-        );
+            w, h, im->image.data, w * 4);
      }
    else
      {
@@ -323,10 +373,7 @@ _evas_common_rgba_image_post_surface(Image_Entry *ie)
 // FIXME: endianess determines this
             PIXMAN_x8r8g8b8,
 //            PIXMAN_b8g8r8x8,
-            im->cache_entry.w, im->cache_entry.h,
-            im->image.data,
-            im->cache_entry.w * 4
-        );
+            w, h, im->image.data, w * 4);
      }
 # else
    (void)ie;
@@ -344,6 +391,9 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned
 
 #ifdef EVAS_CSERVE
    if (ie->data1) return 0;
+#endif
+#ifdef EVAS_CSERVE2
+   if (ie->data1) return 0;
 #endif
    if (im->image.no_free) return 0;
 
@@ -422,6 +472,11 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
    else if (ie->data1)
      evas_cserve_image_free(ie);
 #endif
+// #ifdef EVAS_CSERVE2
+//    else if (ie->data1)
+//      ERR("Shouldn't reach this point since we are using cache2.");
+//      // evas_cserve2_image_free(ie);
+// #endif
 
    im->image.data = NULL;
    ie->allocated.w = 0;
@@ -451,6 +506,10 @@ _evas_common_rgba_image_dirty_region(Image_Entry* ie, unsigned int x __UNUSED__,
 
 #ifdef EVAS_CSERVE
    if (ie->data1) evas_cserve_image_free(ie);
+#endif
+#ifdef EVAS_CSERVE2
+   // if (ie->data1) evas_cserve2_image_free(ie);
+   if (ie->data1) ERR("Shouldn't reach this point since we are using cache2.");
 #endif
    im->flags |= RGBA_IMAGE_IS_DIRTY;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
@@ -471,12 +530,20 @@ _evas_common_rgba_image_dirty(Image_Entry *ie_dst, const Image_Entry *ie_src)
      {
 #ifdef EVAS_CSERVE
         if (ie_src->data1) evas_cserve_image_free((Image_Entry*) ie_src);
+#endif
+#ifdef EVAS_CSERVE2
+        // if (ie_src->data1) evas_cserve2_image_free((Image_Entry*) ie_src);
+        if (ie_src->data1) ERR("Shouldn't reach this point since we are using cache2.");
 #endif
         return 1;
      }
 
 #ifdef EVAS_CSERVE
    if (ie_src->data1) evas_cserve_image_free((Image_Entry*) ie_src);
+#endif
+#ifdef EVAS_CSERVE2
+        // if (ie_src->data1) evas_cserve2_image_free((Image_Entry*) ie_src);
+        if (ie_src->data1) ERR("Shouldn't reach this point since we are using cache2.");
 #endif
    evas_common_image_colorspace_normalize(src);
    evas_common_image_colorspace_normalize(dst);
@@ -498,7 +565,9 @@ _evas_common_rgba_image_ram_usage(Image_Entry *ie)
 
    if (im->image.data)
      {
-#ifdef EVAS_CSERVE
+#if defined(EVAS_CSERVE)
+        if ((!im->image.no_free) || (ie->data1))
+#elif defined(EVAS_CSERVE2)
         if ((!im->image.no_free) || (ie->data1))
 #else
         if ((!im->image.no_free))
@@ -650,6 +719,10 @@ evas_common_image_colorspace_normalize(RGBA_Image *im)
 	  {
 #ifdef EVAS_CSERVE
              if (((Image_Entry *)im)->data1) evas_cserve_image_free(&im->cache_entry);
+#endif
+#ifdef EVAS_CSERVE2
+             // if (((Image_Entry *)im)->data1) evas_cserve2_image_free(&im->cache_entry);
+             if (((Image_Entry *)im)->data1) ERR("Shouldn't reach this point since we are using cache2.");
 #endif
 	     if (!im->image.no_free)
                {
@@ -706,6 +779,16 @@ evas_common_image_colorspace_dirty(RGBA_Image *im)
 {
    im->cs.dirty = 1;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
+#ifdef HAVE_PIXMAN
+# ifdef PIXMAN_IMAGE   
+   if (im->pixman.im)
+     {
+        pixman_image_unref(im->pixman.im);
+        im->pixman.im = NULL;
+     }
+   _evas_common_rgba_image_post_surface((Image_Entry *)im);
+# endif   
+#endif
 }
 
 EAPI void
@@ -713,6 +796,10 @@ evas_common_image_set_cache(unsigned int size)
 {
    if (eci)
      evas_cache_image_set(eci, size);
+#ifdef EVAS_CSERVE2
+   if (eci2)
+     evas_cache2_limit_set(eci2, size);
+#endif
 }
 
 EAPI int
@@ -744,6 +831,14 @@ evas_common_image_cache_get(void)
    return eci;
 }
 
+#ifdef EVAS_CSERVE2
+EAPI Evas_Cache2*
+evas_common_image_cache2_get(void)
+{
+   return eci2;
+}
+#endif
+
 EAPI RGBA_Image *
 evas_common_image_line_buffer_obtain(int len)
 {
