@@ -1,7 +1,7 @@
 #include "evas_common.h"
 
-static RGBA_Gfx_Func     op_blend_span_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
-static RGBA_Gfx_Pt_Func  op_blend_pt_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
+RGBA_Gfx_Func     op_blend_span_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
+RGBA_Gfx_Pt_Func  op_blend_pt_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
 
 static void op_blend_init(void);
 static void op_blend_shutdown(void);
@@ -35,8 +35,8 @@ evas_common_gfx_compositor_blend_get(void)
 }
 
 
-static RGBA_Gfx_Func     op_blend_rel_span_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
-static RGBA_Gfx_Pt_Func  op_blend_rel_pt_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
+RGBA_Gfx_Func     op_blend_rel_span_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
+RGBA_Gfx_Pt_Func  op_blend_rel_pt_funcs[SP_LAST][SM_LAST][SC_LAST][DP_LAST][CPU_LAST];
 
 static void op_blend_rel_init(void);
 static void op_blend_rel_shutdown(void);
@@ -91,11 +91,18 @@ evas_common_gfx_compositor_blend_rel_get(void)
 # include "./evas_op_blend/op_blend_mask_color_neon.c"
 //# include "./evas_op_blend/op_blend_pixel_mask_color_neon.c"
 
+#ifdef BUILD_SSE3
+void evas_common_op_blend_init_sse3(void);
+#endif
+
 static void
 op_blend_init(void)
 {
    memset(op_blend_span_funcs, 0, sizeof(op_blend_span_funcs));
    memset(op_blend_pt_funcs, 0, sizeof(op_blend_pt_funcs));
+#ifdef BUILD_SSE3
+   evas_common_op_blend_init_sse3();
+#endif
 #ifdef BUILD_MMX
    init_blend_pixel_span_funcs_mmx();
    init_blend_pixel_color_span_funcs_mmx();
@@ -121,7 +128,7 @@ op_blend_init(void)
    init_blend_pixel_mask_pt_funcs_neon();
    init_blend_color_pt_funcs_neon();
    init_blend_mask_color_pt_funcs_neon();
-#endif   
+#endif
 #ifdef BUILD_C
    init_blend_pixel_span_funcs_c();
    init_blend_pixel_color_span_funcs_c();
@@ -147,6 +154,14 @@ blend_gfx_span_func_cpu(int s, int m, int c, int d)
 {
    RGBA_Gfx_Func func = NULL;
    int cpu = CPU_N;
+#ifdef BUILD_SSE3
+   if (evas_common_cpu_has_feature(CPU_FEATURE_SSE3))
+      {
+         cpu = CPU_SSE3;
+         func = op_blend_span_funcs[s][m][c][d][cpu];
+         if(func) return func;
+      }
+#endif
 #ifdef BUILD_MMX
    if (evas_common_cpu_has_feature(CPU_FEATURE_MMX))
      {
@@ -263,6 +278,14 @@ blend_gfx_pt_func_cpu(int s, int m, int c, int d)
 {
    RGBA_Gfx_Pt_Func func = NULL;
    int cpu = CPU_N;
+#ifdef BUILD_SSE3
+   if(evas_common_cpu_has_feature(CPU_FEATURE_SSE3))
+      {
+         cpu = CPU_SSE3;
+         func = op_blend_pt_funcs[s][m][c][d][cpu];
+         if(func) return func;
+      }
+#endif
 #ifdef BUILD_MMX
    if (evas_common_cpu_has_feature(CPU_FEATURE_MMX))
      {
@@ -361,12 +384,16 @@ op_blend_pixel_mask_pt_get(Image_Entry_Flags src_flags, RGBA_Image *dst)
    return blend_gfx_pt_func_cpu(s, m, c, d);
 }
 
+void evas_common_op_blend_rel_init_sse3(void);
 
 static void
 op_blend_rel_init(void)
 {
    memset(op_blend_rel_span_funcs, 0, sizeof(op_blend_rel_span_funcs));
    memset(op_blend_rel_pt_funcs, 0, sizeof(op_blend_rel_pt_funcs));
+#ifdef BUILD_SSE3
+   evas_common_op_blend_rel_init_sse3();
+#endif
 #ifdef BUILD_MMX
    init_blend_rel_pixel_span_funcs_mmx();
    init_blend_rel_pixel_color_span_funcs_mmx();
@@ -418,6 +445,14 @@ blend_rel_gfx_span_func_cpu(int s, int m, int c, int d)
 {
    RGBA_Gfx_Func func = NULL;
    int cpu = CPU_N;
+#ifdef BUILD_SSE3
+   if (evas_common_cpu_has_feature(CPU_FEATURE_SSE3))
+      {
+         cpu = CPU_SSE3;
+         func = op_blend_rel_span_funcs[s][m][c][d][cpu];
+         if(func) return func;
+      }
+#endif
 #ifdef BUILD_MMX
    if (evas_common_cpu_has_feature(CPU_FEATURE_MMX))
      {
@@ -529,6 +564,14 @@ blend_rel_gfx_pt_func_cpu(int s, int m, int c, int d)
 {
    RGBA_Gfx_Pt_Func func = NULL;
    int cpu = CPU_N;
+#ifdef BUILD_SSE3
+   if (evas_common_cpu_has_feature(CPU_FEATURE_SSE3))
+      {
+         cpu = CPU_SSE3;
+         func = op_blend_rel_pt_funcs[s][m][c][d][cpu];
+         if(func) return func;
+      }
+#endif
 #ifdef BUILD_MMX
    if (evas_common_cpu_has_feature(CPU_FEATURE_MMX))
      {
