@@ -3,9 +3,6 @@
    RGBA_Gfx_Func func;
    RGBA_Image *maskobj;
    DATA8 *mask = NULL;
-#ifdef EVAS_SLI
-   int ysli = dst_clip_y;
-#endif
 
    ptr = src->image.data + ((dst_clip_y - dst_region_y + src_region_y) * src_w) + (dst_clip_x - dst_region_x) + src_region_x;
    if (dc->mask.mask)
@@ -25,36 +22,21 @@
      //   mask += (dst_clip_y - dc->mask.y) * maskobj->cache_entry.w;
         while (dst_clip_h--)
           {
+	    func(ptr, mask, dc->mul.col, dst_ptr, dst_clip_w);
 
-#ifdef EVAS_SLI
-             if (((ysli) % dc->sli.h) == dc->sli.y)
-#endif
-               {
-                  func(ptr, mask, dc->mul.col, dst_ptr, dst_clip_w);
-               }
-#ifdef EVAS_SLI
-             ysli++;
-#endif
-             ptr += src_w;
-             dst_ptr += dst_w;
-             mask += maskobj->cache_entry.w;
+	    ptr += src_w;
+	    dst_ptr += dst_w;
+	    mask += maskobj->cache_entry.w;
           }
      }
    else
      {
         while (dst_clip_h--)
           {
-#ifdef EVAS_SLI
-             if (((ysli) % dc->sli.h) == dc->sli.y)
-#endif
-               {
-                  func(ptr, NULL, dc->mul.col, dst_ptr, dst_clip_w);
-               }
-#ifdef EVAS_SLI
-             ysli++;
-#endif
-             ptr += src_w;
-             dst_ptr += dst_w;
+	    func(ptr, NULL, dc->mul.col, dst_ptr, dst_clip_w);
+
+	    ptr += src_w;
+	    dst_ptr += dst_w;
           }
      }
 }
