@@ -474,8 +474,8 @@ evas_common_soft16_image_draw(Soft16_Image *src, Soft16_Image *dst,
 		  int dst_region_w, int dst_region_h,
 		  int smooth __UNUSED__)
 {
+   static Cutout_Rects *rects = NULL;
    Eina_Rectangle sr, dr;
-   Cutout_Rects *rects;
    Cutout_Rect  *r;
    struct RGBA_Draw_Context_clip clip_bkp;
    int i;
@@ -510,14 +510,13 @@ evas_common_soft16_image_draw(Soft16_Image *src, Soft16_Image *dst,
 	dc->clip = clip_bkp;
 	return;
      }
-   rects = evas_common_draw_context_apply_cutouts(dc);
+   rects = evas_common_draw_context_apply_cutouts(dc, rects);
    for (i = 0; i < rects->active; i++)
      {
 	r = rects->rects + i;
 	evas_common_draw_context_set_clip(dc, r->x, r->y, r->w, r->h);
 	_soft16_image_draw_sampled_int(src, dst, dc, sr, dr);
      }
-   evas_common_draw_context_apply_clear_cutouts(rects);
    dc->clip = clip_bkp;
 }
 
