@@ -78,8 +78,8 @@ void
 evas_common_soft16_rectangle_draw(Soft16_Image *dst, RGBA_Draw_Context *dc,
                       int x, int y, int w, int h)
 {
+   static Cutout_Rects *rects = NULL;
    Eina_Rectangle dr;
-   Cutout_Rects *rects;
    Cutout_Rect  *r;
    struct RGBA_Draw_Context_clip c_bkp;
    int i;
@@ -108,14 +108,13 @@ evas_common_soft16_rectangle_draw(Soft16_Image *dst, RGBA_Draw_Context *dc,
 	dc->clip = c_bkp;
 	return;
      }
-   rects = evas_common_draw_context_apply_cutouts(dc);
+   rects = evas_common_draw_context_apply_cutouts(dc, rects);
    for (i = 0; i < rects->active; ++i)
      {
 	r = rects->rects + i;
 	evas_common_draw_context_set_clip(dc, r->x, r->y, r->w, r->h);
         _soft16_rectangle_draw_int(dst, dc, dr);
      }
-   evas_common_draw_context_apply_clear_cutouts(rects);
    dc->clip = c_bkp;
 }
 
