@@ -42,7 +42,7 @@ _evas_module_append(Eina_List *list, char *path)
 void
 evas_module_paths_init(void)
 {
-   char *path;
+   char *libdir, *path;
 
    /* 1. ~/.evas/modules/ */
    path = eina_module_environment_path_get("HOME", "/.evas/modules");
@@ -56,13 +56,24 @@ evas_module_paths_init(void)
      evas_module_paths = _evas_module_append(evas_module_paths, path);
 
    /* 3. libevas.so/../evas/modules/ */
-   path = eina_module_symbol_path_get(evas_module_paths_init, "/evas/modules");
+   libdir = (char *)_evas_module_libdir_get();
+   if (!libdir)
+     path = eina_module_symbol_path_get(evas_module_paths_init, "/evas/modules");
+   else
+     {
+        path = malloc(strlen(libdir) + strlen("/evas/modules") + 1);
+        if (path)
+          {
+             strcpy(path, libdir);
+             strcat(path, "/evas/modules");
+          }
+     }
    if (eina_list_search_unsorted(evas_module_paths, (Eina_Compare_Cb) strcmp, path))
      free(path);
    else
      evas_module_paths = _evas_module_append(evas_module_paths, path);
 
-   /* 4. PREFIX/evas/modules/ */
+   /* 4. PREFIX/lib/evas/modules/ */
 #ifndef _MSC_VER
    path = PACKAGE_LIB_DIR "/evas/modules";
    if (!eina_list_search_unsorted(evas_module_paths, (Eina_Compare_Cb) strcmp, path))
@@ -89,7 +100,6 @@ EVAS_EINA_STATIC_MODULE_DEFINE(engine, gl_x11);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, gl_sdl);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, psl1ght);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16_ddraw);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16_sdl);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16_wince);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16_x11);
@@ -148,9 +158,6 @@ static const struct {
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_16
   EVAS_EINA_STATIC_MODULE_USE(engine, software_16),
 #endif
-#ifdef EVAS_STATIC_BUILD_SOFTWARE_16_DDRAW
-  EVAS_EINA_STATIC_MODULE_USE(engine, software_16_ddraw),
-#endif
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_16_SDL
   EVAS_EINA_STATIC_MODULE_USE(engine, software_16_sdl),
 #endif
@@ -160,18 +167,24 @@ static const struct {
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_16_X11
   EVAS_EINA_STATIC_MODULE_USE(engine, software_16_x11),
 #endif
-#ifdef EVAS_STATIC_BUILD_SOFTWARE_16_DDRAW
-  EVAS_EINA_STATIC_MODULE_USE(engine, software_ddraw),
-#endif
-#ifdef EVAS_STATIC_BUILD_SOFTWARE_16_GDI
+#ifdef EVAS_STATIC_BUILD_SOFTWARE_GDI
   EVAS_EINA_STATIC_MODULE_USE(engine, software_gdi),
 #endif
+#ifdef EVAS_STATIC_BUILD_SOFTWARE_DDRAW
+  EVAS_EINA_STATIC_MODULE_USE(engine, software_ddraw),
+#endif
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_8
   EVAS_EINA_STATIC_MODULE_USE(engine, software_8),
 #endif
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_8_X11
   EVAS_EINA_STATIC_MODULE_USE(engine, software_8_x11),
 #endif
+#ifdef EVAS_STATIC_BUILD_SOFTWARE_DDRAW
+  EVAS_EINA_STATIC_MODULE_USE(engine, software_ddraw),
+#endif
+#ifdef EVAS_STATIC_BUILD_SOFTWARE_GDI
+  EVAS_EINA_STATIC_MODULE_USE(engine, software_gdi),
+#endif
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_GENERIC
   EVAS_EINA_STATIC_MODULE_USE(engine, software_generic),
 #endif
@@ -558,6 +571,8 @@ evas_module_clean(void)
 /*      } */
 }
 
+static Eina_Prefix *pfx = NULL;
+
 /* will dlclose all the modules loaded and free all the structs */
 void
 evas_module_shutdown(void)
@@ -586,6 +601,11 @@ evas_module_shutdown(void)
 
    eina_array_free(evas_engines);
    evas_engines = NULL;
+   if (pfx)
+     {
+        eina_prefix_free(pfx);
+        pfx = NULL;
+     }
 }
 
 EAPI int
@@ -608,13 +628,11 @@ _evas_module_engine_inherit(Evas_Func *funcs, char *name)
    return 0;
 }
 
-static Eina_Prefix *pfx = NULL;
-
 EAPI const char *
 _evas_module_libdir_get(void)
 {
    if (!pfx) pfx = eina_prefix_new
-      (NULL, _evas_module_libdir_get, "EVAS", "evas", NULL,
+      (NULL, _evas_module_libdir_get, "EVAS", "evas", "checkme",
        PACKAGE_BIN_DIR, PACKAGE_LIB_DIR, PACKAGE_DATA_DIR, PACKAGE_DATA_DIR);
    if (!pfx) return NULL;
    return eina_prefix_lib_get(pfx);
