@@ -8,14 +8,6 @@
 extern "C" {
 #endif
 
-#ifndef always_inline
-#if defined(__GNUC__) && (__GNUC__ > 3 || __GNUC__ == 3 && __GNUC_MINOR__ > 0)
-#    define always_inline __attribute__((always_inline)) inline
-#else
-#    define always_inline inline
-#endif
-#endif
-
 #define RGB_565_UNPACKED_MASK 0x07e0f81f
 #define RGB_565_UNPACK(rgb)                                             \
    (((rgb) | ((rgb) << 16)) & RGB_565_UNPACKED_MASK)
