@@ -400,7 +400,7 @@ Direct3DImageHandler evas_direct3d_image_load(Direct3DDeviceHandler d3d,
    D3DDevice *device = dev_ptr->device;
    D3DScene *scene = dev_ptr->scene;
 
-   RGBA_Image *evas_image = evas_common_load_image_from_file(file, key, lo);
+   RGBA_Image *evas_image = evas_common_load_image_from_file(file, key, lo, error);
    if (evas_image == NULL)
    {
       WRN("Failed to load image from %s", file);
