@@ -209,12 +209,12 @@ void D3DObjectImage::Setup(FLOAT x, FLOAT y, FLOAT w, FLOAT h,
    }
    else
    {
-      _x = min(_x, x);
-      _y = max(_y, y);
+      _x = __min(_x, x);
+      _y = __max(_y, y);
       _w += w / 3;
       _h += h / 3;
-      _sx = min(_sx, FLOAT(sx) / FLOAT(_width));
-      _sy = min(_sy, FLOAT(sy) / FLOAT(_height));
+      _sx = __min(_sx, FLOAT(sx) / FLOAT(_width));
+      _sy = __min(_sy, FLOAT(sy) / FLOAT(_height));
       _sw += FLOAT(sw) / (3.f * FLOAT(_width));
       _sh += FLOAT(sh) / (3.f * FLOAT(_height));
    }
