@@ -2,7 +2,7 @@
 #define _EVAS_ENGINE_DIRECTFB_H
 
 #include <Evas.h>
-#include <directfb/directfb.h>
+#include <directfb.h>
 
 typedef struct _Evas_Engine_Info_DirectFB Evas_Engine_Info_DirectFB;
 
