@@ -108,6 +108,9 @@
 #ifndef GL_PROGRAM_BINARY_RETRIEVABLE_HINT
 # define GL_PROGRAM_BINARY_RETRIEVABLE_HINT 0x8257
 #endif
+#ifndef GL_MAX_SAMPLES_IMG
+#define GL_MAX_SAMPLES_IMG 0x9135
+#endif
 
 #define SHAD_VERTEX 0
 #define SHAD_COLOR  1
@@ -281,7 +284,7 @@ struct _Evas_Engine_GL_Context
       int                top_pipe;
       struct {
          GLuint          cur_prog;
-	 GLuint          cur_tex, cur_texu, cur_texv;
+         GLuint          cur_tex, cur_texu, cur_texv;
          GLuint          cur_texm, cur_texmu, cur_texmv;
          int             render_op;
          int             cx, cy, cw, ch;
@@ -304,8 +307,8 @@ struct _Evas_Engine_GL_Context
          Evas_GL_Image  *surface;
          GLuint          cur_prog;
          GLuint          cur_tex, cur_texu, cur_texv, cur_texm;
-	 void           *cur_tex_dyn, *cur_texu_dyn, *cur_texv_dyn;
-	 int             render_op;
+         void           *cur_tex_dyn, *cur_texu_dyn, *cur_texv_dyn;
+         int             render_op;
          int             cx, cy, cw, ch;
          int             smooth;
          int             blend;
@@ -319,7 +322,7 @@ struct _Evas_Engine_GL_Context
          GLfloat *texuv2;
          GLfloat *texuv3;
          GLfloat *texm;
-	 Eina_Bool line: 1;
+         Eina_Bool line: 1;
          Eina_Bool use_vertex : 1;
          Eina_Bool use_color : 1;
          Eina_Bool use_texuv : 1;
@@ -333,15 +336,17 @@ struct _Evas_Engine_GL_Context
    struct {
       Eina_Bool size : 1;
    } change;
+   
+   Eina_List *font_glyph_textures;
 
    Eina_Bool havestuff : 1;
 
    Evas_GL_Image *def_surface;
 
    /* If this is set: Force drawing with a particular filter */
-   GLuint	filter_prog;
+   GLuint filter_prog;
 
-#if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+#if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX) || defined(BUILD_ENGINE_WAYLAND_EGL)
 // FIXME: hack. expose egl display to gl core for egl image sec extn.
    void *egldisp;
 #endif
@@ -374,6 +379,7 @@ struct _Evas_GL_Texture
    Evas_Engine_GL_Context *gc;
    Evas_GL_Image   *im;
    Evas_GL_Texture_Pool *pt, *ptu, *ptv, *ptuv;
+   RGBA_Font_Glyph *fglyph;
    int              x, y, w, h;
    double           sx1, sy1, sx2, sy2;
    int              references;
@@ -540,17 +546,17 @@ void             evas_gl_common_context_yuv_push(Evas_Engine_GL_Context *gc,
                                                  int r, int g, int b, int a,
                                                  Eina_Bool smooth);
 void             evas_gl_common_context_yuy2_push(Evas_Engine_GL_Context *gc,
-						  Evas_GL_Texture *tex,
-						  double sx, double sy, double sw, double sh,
-						  int x, int y, int w, int h,
-						  int r, int g, int b, int a,
-						  Eina_Bool smooth);
+                                                  Evas_GL_Texture *tex,
+                                                  double sx, double sy, double sw, double sh,
+                                                  int x, int y, int w, int h,
+                                                  int r, int g, int b, int a,
+                                                  Eina_Bool smooth);
 void             evas_gl_common_context_nv12_push(Evas_Engine_GL_Context *gc,
-						  Evas_GL_Texture *tex,
-						  double sx, double sy, double sw, double sh,
-						  int x, int y, int w, int h,
-						  int r, int g, int b, int a,
-						  Eina_Bool smooth);
+                                                  Evas_GL_Texture *tex,
+                                                  double sx, double sy, double sw, double sh,
+                                                  int x, int y, int w, int h,
+                                                  int r, int g, int b, int a,
+                                                  Eina_Bool smooth);
 void             evas_gl_common_context_image_map_push(Evas_Engine_GL_Context *gc,
                                                        Evas_GL_Texture *tex,
                                                        int npoints,
@@ -559,7 +565,7 @@ void             evas_gl_common_context_image_map_push(Evas_Engine_GL_Context *g
                                                        int r, int g, int b, int a,
                                                        Eina_Bool smooth,
                                                        Eina_Bool tex_only,
-						       Evas_Colorspace cspace);
+                                                       Evas_Colorspace cspace);
 void              evas_gl_common_context_flush(Evas_Engine_GL_Context *gc);
 
 int               evas_gl_common_shader_program_init(Evas_GL_Shared *shared);
