@@ -718,7 +718,7 @@ evas_gl_common_context_new(void)
         SHADER_TEXTURE_ADD(shared, IMG_MASK, texm);
 
         if (gc->state.current.cur_prog == PRG_INVALID)
-           glUseProgram(gc->shared->shader[0].prog);
+           glUseProgram(shared->shader[0].prog);
         else glUseProgram(gc->state.current.cur_prog);
         GLERR(__FUNCTION__, __FILE__, __LINE__, "");
 
@@ -770,6 +770,9 @@ evas_gl_common_context_free(Evas_Engine_GL_Context *gc)
              if (gc->pipe[i].array.texuv3) free(gc->pipe[i].array.texuv3);
           }
      }
+   
+   while (gc->font_glyph_textures)
+     evas_gl_common_texture_free(gc->font_glyph_textures->data);
 
    if ((gc->shared) && (gc->shared->references == 0))
      {
@@ -783,16 +786,18 @@ evas_gl_common_context_free(Evas_Engine_GL_Context *gc)
              evas_gl_common_image_free(gc->shared->images->data);
           }
 
-        EINA_LIST_FOREACH(gc->shared->tex.whole, l, pt)
-           evas_gl_texture_pool_empty(pt);
         for (i = 0; i < 33; i++)
           {
              for (j = 0; j < 3; j++)
                {
                   EINA_LIST_FOREACH(gc->shared->tex.atlas[i][j], l, pt)
                      evas_gl_texture_pool_empty(pt);
+                  eina_list_free(gc->shared->tex.atlas[i][j]);
                }
           }
+        EINA_LIST_FOREACH(gc->shared->tex.whole, l, pt)
+           evas_gl_texture_pool_empty(pt);
+        eina_list_free(gc->shared->tex.whole);
         eina_hash_free(gc->shared->native_pm_hash);
         eina_hash_free(gc->shared->native_tex_hash);
         free(gc->shared);
@@ -2058,8 +2063,8 @@ evas_gl_common_context_image_map_push(Evas_Engine_GL_Context *gc,
                                       RGBA_Map_Point *p,
                                       int clip, int cx, int cy, int cw, int ch,
                                       int r, int g, int b, int a,
-				      Eina_Bool smooth, Eina_Bool tex_only,
-				      Evas_Colorspace cspace)
+                                      Eina_Bool smooth, Eina_Bool tex_only,
+                                      Evas_Colorspace cspace)
 {
    int pnum, nv, nc, nu, nu2, nu3, nt, i;
    const int points[6] = { 0, 1, 2, 0, 2, 3 };
@@ -2183,7 +2188,7 @@ evas_gl_common_context_image_map_push(Evas_Engine_GL_Context *gc,
           {
              clip = 0; cx = 0; cy = 0; cw = 0; ch = 0;
           }
-        x = nx; y = nw; w = nw; h = nh;
+        x = nx; y = ny; w = nw; h = nh;
      }
 
    if (!flat)
@@ -2198,12 +2203,12 @@ evas_gl_common_context_image_map_push(Evas_Engine_GL_Context *gc,
      }
 
    pn = _evas_gl_common_context_push(RTYPE_MAP,
-				     gc, tex, NULL,
-				     prog,
-				     x, y, w, h,
-				     blend,
-				     smooth,
-				     clip, cx, cy, cw, ch);
+                                     gc, tex, NULL,
+                                     prog,
+                                     x, y, w, h,
+                                     blend,
+                                     smooth,
+                                     clip, cx, cy, cw, ch);
    gc->pipe[pn].region.type = RTYPE_MAP;
    gc->pipe[pn].shader.cur_tex = tex->pt->texture;
    if (utexture)
