@@ -314,6 +314,7 @@ evas_gl_common_image_alpha_set(Evas_GL_Image *im, int alpha)
    if (im->alpha == alpha) return im;
    im->alpha = alpha;
    if (!im->im) return im;
+   evas_cache_image_load_data(&im->im->cache_entry);
    im->im->cache_entry.flags.alpha = alpha ? 1 : 0;
    if (im->tex)
      {
@@ -321,7 +322,16 @@ evas_gl_common_image_alpha_set(Evas_GL_Image *im, int alpha)
         im->tex = NULL;
      }
    if (!im->tex)
-     im->tex = evas_gl_common_texture_new(im->gc, im->im);
+     {
+        if (im->tex_only)
+          im->tex = evas_gl_common_texture_native_new(im->gc, im->w, im->h,
+                                                      im->alpha, im);
+        else
+          {
+             im->tex = evas_gl_common_texture_new(im->gc, im->im);
+             evas_gl_common_texture_update(im->tex, im->im);
+          }
+     }
    return im;
 }
 
@@ -662,6 +672,7 @@ evas_gl_common_image_map_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im,
    int c, cx, cy, cw, ch;
 
    dc = gc->dc;
+
    if (dc->mul.use)
      {
         a = (dc->mul.col >> 24) & 0xff;
@@ -676,9 +687,9 @@ evas_gl_common_image_map_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im,
 
    evas_gl_common_image_update(gc, im);
 
-   c = gc->dc->clip.use;
-   cx = gc->dc->clip.x; cy = gc->dc->clip.y;
-   cw = gc->dc->clip.w; ch = gc->dc->clip.h;
+   c = dc->clip.use;
+   cx = dc->clip.x;   cy = dc->clip.y;
+   cw = dc->clip.w;   ch = dc->clip.h;
    im->tex->im = im;
 
    evas_gl_common_context_image_map_push(gc, im->tex, npoints, p,
@@ -690,12 +701,12 @@ evas_gl_common_image_map_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im,
 void
 evas_gl_common_image_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh, int smooth)
 {
+   static Cutout_Rects *rects = NULL;
    RGBA_Draw_Context *dc;
    Evas_GL_Image *imm;
    int r, g, b, a;
    double ssx, ssy, ssw, ssh;
    double mssx, mssy, mssw, mssh;
-   Cutout_Rects *rects;
    Cutout_Rect  *rct;
    int c, cx, cy, cw, ch;
    int i;
@@ -888,7 +899,7 @@ evas_gl_common_image_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im, int sx,
         gc->dc->clip.use = c; gc->dc->clip.x = cx; gc->dc->clip.y = cy; gc->dc->clip.w = cw; gc->dc->clip.h = ch;
         return;
      }
-   rects = evas_common_draw_context_apply_cutouts(dc);
+   rects = evas_common_draw_context_apply_cutouts(dc, rects);
    for (i = 0; i < rects->active; ++i)
      {
         int nx, ny, nw, nh;
@@ -962,7 +973,6 @@ evas_gl_common_image_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im, int sx,
                                             r, g, b, a,
                                             smooth, im->tex_only);
      }
-   evas_common_draw_context_apply_clear_cutouts(rects);
    /* restore clip info */
    gc->dc->clip.use = c; gc->dc->clip.x = cx; gc->dc->clip.y = cy; gc->dc->clip.w = cw; gc->dc->clip.h = ch;
 }
