@@ -1,6 +1,10 @@
 "#ifdef GL_ES\n"
+"#ifdef GL_FRAGMENT_PRECISION_HIGH\n"
+"precision highp float;\n"
+"#else\n"
 "precision mediump float;\n"
 "#endif\n"
+"#endif\n"
 "uniform sampler2D tex;\n"
 "varying vec2 tex_c;\n"
 "void main()\n"
