@@ -24,13 +24,13 @@ static int win_count = 0;
 
 Evas_GL_X11_Window *
 eng_window_new(Display *disp,
-	       Window   win,
-	       int      screen,
-	       Visual  *vis,
-	       Colormap cmap,
-	       int      depth,
-	       int      w,
-	       int      h,
+               Window   win,
+               int      screen,
+               Visual  *vis,
+               Colormap cmap,
+               int      depth,
+               int      w,
+               int      h,
                int      indirect,
                int      alpha,
                int      rot)
@@ -44,6 +44,7 @@ eng_window_new(Display *disp,
 #endif
    XVisualInfo *vi_use;
    const GLubyte *vendor, *renderer, *version;
+   int blacklist = 0;
 
    if (!_evas_gl_x11_vi) return NULL;
 
@@ -160,20 +161,20 @@ eng_window_new(Display *disp,
    if (!gw->egl_disp)
      {
         ERR("eglGetDisplay() fail. code=%#x", eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
    if (!eglInitialize(gw->egl_disp, &major_version, &minor_version))
      {
         ERR("eglInitialize() fail. code=%#x", eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
    eglBindAPI(EGL_OPENGL_ES_API);
    if (eglGetError() != EGL_SUCCESS)
      {
         ERR("eglBindAPI() fail. code=%#x", eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
 
@@ -182,7 +183,7 @@ eng_window_new(Display *disp,
                         1, &num_config) || (num_config != 1))
      {
         ERR("eglChooseConfig() fail. code=%#x", eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
    gw->egl_surface[0] = eglCreateWindowSurface(gw->egl_disp, gw->egl_config,
@@ -192,26 +193,27 @@ eng_window_new(Display *disp,
      {
         ERR("eglCreateWindowSurface() fail for %#x. code=%#x",
             (unsigned int)gw->win, eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
-   if (context == EGL_NO_CONTEXT)
-     context = eglCreateContext(gw->egl_disp, gw->egl_config, NULL,
-                                context_attrs);
-   gw->egl_context[0] = context;
+   
+   gw->egl_context[0] = eglCreateContext
+     (gw->egl_disp, gw->egl_config, context, context_attrs);
    if (gw->egl_context[0] == EGL_NO_CONTEXT)
      {
         ERR("eglCreateContext() fail. code=%#x", eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
+   if (context == EGL_NO_CONTEXT) context = gw->egl_context[0];
+   
    if (eglMakeCurrent(gw->egl_disp,
                       gw->egl_surface[0],
                       gw->egl_surface[0],
                       gw->egl_context[0]) == EGL_FALSE)
      {
         ERR("eglMakeCurrent() fail. code=%#x", eglGetError());
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
 
@@ -227,6 +229,23 @@ eng_window_new(Display *disp,
         fprintf(stderr, "renderer: %s\n", renderer);
         fprintf(stderr, "version: %s\n", version);
      }
+
+   if (strstr((const char *)vendor, "Mesa Project"))
+     {
+        if (strstr((const char *)renderer, "Software Rasterizer"))
+          blacklist = 1;
+     }
+   if (strstr((const char *)renderer, "softpipe"))
+     blacklist = 1;
+   if (blacklist)
+     {
+        ERR("OpenGL Driver blacklisted:");
+        ERR("Vendor: %s", (const char *)vendor);
+        ERR("Renderer: %s", (const char *)renderer);
+        ERR("Version: %s", (const char *)version);
+        eng_window_free(gw);
+        return NULL;
+     }
 // GLX
 #else
    if (!context)
@@ -265,7 +284,7 @@ eng_window_new(Display *disp,
      gw->glxwin = glXCreateWindow(gw->disp, fbconf, gw->win, NULL);
    if (!gw->glxwin)
      {
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
 
@@ -277,15 +296,11 @@ eng_window_new(Display *disp,
 
    if (!gw->context)
      {
-	eng_window_free(gw);
+        eng_window_free(gw);
         return NULL;
      }
    if (gw->context)
      {
-        int i, j,  num;
-        GLXFBConfig *fbc;
-        int blacklist = 0;
-
         if (gw->glxwin)
           {
              if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin,
@@ -415,96 +430,14 @@ eng_window_new(Display *disp,
           {
              // noothing yet. add more cases and options over time
           }
-
-        fbc = glXGetFBConfigs(gw->disp, screen, &num);
-        if (!fbc)
-          {
-             ERR("glXGetFBConfigs() returned no fb configs");
-             eng_window_free(gw);
-             return NULL;
-          }
-        for (i = 0; i <= 32; i++)
-          {
-             for (j = 0; j < num; j++)
-               {
-                  XVisualInfo *vi;
-                  int vd;
-                  int alph, val, dbuf, stencil, tdepth;
-                  int rgba;
-
-                  vi = glXGetVisualFromFBConfig(gw->disp, fbc[j]);
-                  if (!vi) continue;
-                  vd = vi->depth;
-                  XFree(vi);
-
-                  if (vd != i) continue;
-
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_ALPHA_SIZE, &alph);
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BUFFER_SIZE, &val);
-
-                  if ((val != i) && ((val - alph) != i)) continue;
-
-                  val = 0;
-                  rgba = 0;
-
-                  if (i == 32)
-                    {
-                       glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_TEXTURE_RGBA_EXT, &val);
-                       if (val)
-                         {
-                            rgba = 1;
-                            gw->depth_cfg[i].tex_format = GLX_TEXTURE_FORMAT_RGBA_EXT;
-                         }
-                    }
-                  if (!val)
-                    {
-                       if (rgba) continue;
-                       glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_TEXTURE_RGB_EXT, &val);
-                       if (!val) continue;
-                       gw->depth_cfg[i].tex_format = GLX_TEXTURE_FORMAT_RGB_EXT;
-                    }
-
-                  dbuf = 0x7fff;
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_DOUBLEBUFFER, &val);
-                  if (val > dbuf) continue;
-                  dbuf = val;
-
-                  stencil = 0x7fff;
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_STENCIL_SIZE, &val);
-                  if (val > stencil) continue;
-                  stencil = val;
-
-                  tdepth = 0x7fff;
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_DEPTH_SIZE, &val);
-                  if (val > tdepth) continue;
-                  tdepth = val;
-
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_MIPMAP_TEXTURE_EXT, &val);
-                  if (val < 0) continue;
-                  gw->depth_cfg[i].mipmap = val;
-
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_Y_INVERTED_EXT, &val);
-                  gw->depth_cfg[i].yinvert = val;
-
-                  glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_TEXTURE_TARGETS_EXT, &val);
-                  gw->depth_cfg[i].tex_target = val;
-
-                  gw->depth_cfg[i].fbc = fbc[j];
-               }
-          }
-        XFree(fbc);
-        if (!gw->depth_cfg[DefaultDepth(gw->disp, screen)].fbc)
-          {
-             WRN("texture from pixmap not going to work");
-          }
      }
 #endif
 
    gw->gl_context = evas_gl_common_context_new();
    if (!gw->gl_context)
      {
-	eng_window_free(gw);
-	return NULL;
+        eng_window_free(gw);
+        return NULL;
      }
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
    gw->gl_context->egldisp = gw->egl_disp;
@@ -529,9 +462,11 @@ eng_window_free(Evas_GL_X11_Window *gw)
         evas_gl_common_context_free(gw->gl_context);
      }
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
+   eglMakeCurrent(gw->egl_disp, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
    if (gw->egl_surface[0] != EGL_NO_SURFACE)
       eglDestroySurface(gw->egl_disp, gw->egl_surface[0]);
-   eglMakeCurrent(gw->egl_disp, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
+   if (gw->egl_context[0] != context)
+     eglDestroyContext(gw->egl_disp, gw->egl_context[0]);
    if (ref == 0)
      {
         if (context) eglDestroyContext(gw->egl_disp, context);
@@ -793,7 +728,7 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
              config_attrs[i++] = GLX_TRANSPARENT_TYPE;
              config_attrs[i++] = GLX_NONE;//GLX_NONE;//GLX_TRANSPARENT_INDEX//GLX_TRANSPARENT_RGB;
              config_attrs[i++] = 0;
-             
+
              configs = glXChooseFBConfig(einfo->info.display,
                                          einfo->info.screen,
                                          config_attrs, &num);
