@@ -292,7 +292,7 @@ eng_image_cache_get(void *data __UNUSED__)
 }
 
 static void
-eng_font_draw(void *data __UNUSED__, void *context, void *surface, Evas_Font_Set *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const Evas_Text_Props *text_props)
+eng_font_draw(void *data __UNUSED__, void *context, void *surface, Evas_Font_Set *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, Evas_Text_Props *text_props)
 {
    static RGBA_Image    *im = NULL;
    Soft16_Image         *dst = surface;
@@ -305,7 +305,8 @@ eng_font_draw(void *data __UNUSED__, void *context, void *surface, Evas_Font_Set
 					 evas_common_soft16_font_glyph_new,
 					 evas_common_soft16_font_glyph_free,
 					 evas_common_soft16_font_glyph_draw);
-   evas_common_font_draw(im, context, (RGBA_Font *) font, x, y, text_props);
+   evas_common_font_draw_prepare(text_props);
+   evas_common_font_draw(im, context, x, y, text_props);
    evas_common_draw_context_font_ext_set(context,
 					 NULL,
 					 NULL,
