@@ -160,7 +160,6 @@ eng_setup(Evas *e, void *in)
 	evas_common_convert_init();
 	evas_common_scale_init();
 	evas_common_rectangle_init();
-	evas_common_gradient_init();
 	evas_common_polygon_init();
 	evas_common_line_init();
 	evas_common_font_init();
@@ -716,11 +715,11 @@ module_open(Evas_Module *em)
    if (!em) return 0;
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&pfunc, "software_16")) return 0;
-   _evas_engine_soft16_wince_log_dom = eina_log_domain_register("EvasSoft16Wince", EVAS_DEFAULT_LOG_COLOR);
-   
-   if(_evas_engine_soft16_wince_log_dom < 0)
+   _evas_engine_soft16_wince_log_dom = eina_log_domain_register
+     ("evas-software_16_wince", EVAS_DEFAULT_LOG_COLOR);
+   if (_evas_engine_soft16_wince_log_dom < 0)
      {
-        EINA_LOG_ERR("Impossible to create a log domain for the Soft16 Wince engine.");
+        EINA_LOG_ERR("Can not create a module log domain.");
         return 0;
      }
 
