@@ -82,7 +82,7 @@ struct _Outbuf
                   Pixmap mask;
                   Visual *vis;
                   Colormap cmap;
-                  int depth, shm;
+                  int depth, imdepth, shm;
                   GC gc, gcm;
                   unsigned char swap : 1;
                   unsigned char bit_swap : 1;
@@ -97,7 +97,7 @@ struct _Outbuf
                   xcb_pixmap_t mask;
                   xcb_visualtype_t *visual;
                   xcb_colormap_t cmap;
-                  int depth, shm;
+                  int depth, imdepth, shm;
                   xcb_gcontext_t gc, gcm;
                   unsigned char swap : 1;
                   unsigned char bit_swap : 1;
@@ -111,7 +111,7 @@ struct _Outbuf
 
         /* 1 big buffer for updates - flush on idle_flush */
         RGBA_Image *onebuf;
-        Eina_List *onebuf_regions;
+        Eina_Array  onebuf_regions;
 
         /* a list of pending regions to write to the target */
         Eina_List *pending_writes;
@@ -119,11 +119,6 @@ struct _Outbuf
         /* a list of previous frame pending regions to write to the target */
         Eina_List *prev_pending_writes;
 
-# ifdef EVAS_FRAME_QUEUING
-        /* protecting prev_pending_writes */
-        LK(lock);
-# endif
-
         unsigned char mask_dither : 1;
         unsigned char destination_alpha : 1;
         unsigned char debug : 1;
