@@ -5,9 +5,6 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-#include <unistd.h>
 
 #ifdef HAVE_EVIL
 # include <Evil.h>
@@ -31,7 +28,10 @@
 
 #define TGA_SIGNATURE "TRUEVISION-XFILE"
 
-typedef struct
+typedef struct _tga_header tga_header;
+typedef struct _tga_footer tga_footer;
+
+struct _tga_header
 {
    unsigned char       idLength;
    unsigned char       colorMapType;
@@ -45,16 +45,16 @@ typedef struct
    unsigned char       heightLo, heightHi;
    unsigned char       bpp;
    unsigned char       descriptor;
-} tga_header;
-                  
-typedef struct
+} __attribute__((packed));
+
+struct _tga_footer
 {
    unsigned int        extensionAreaOffset;
    unsigned int        developerDirectoryOffset;
    char                signature[16];
    char                dot;
    char                null;
-} tga_footer;
+} __attribute__((packed));
 
 
 static Eina_Bool evas_image_load_file_head_tga(Image_Entry *ie, const char *file, const char *key, int *error) EINA_ARG_NONNULL(1, 2, 4);
@@ -64,46 +64,53 @@ static Evas_Image_Load_Func evas_image_load_tga_func =
 {
   EINA_TRUE,
   evas_image_load_file_head_tga,
-  evas_image_load_file_data_tga
+  evas_image_load_file_data_tga,
+  NULL,
+  EINA_FALSE
 };
 
 static Eina_Bool
 evas_image_load_file_head_tga(Image_Entry *ie, const char *file, const char *key __UNUSED__, int *error)
 {
-   int fd;
-   unsigned char *seg = MAP_FAILED, *filedata;
-   struct stat ss;
+   Eina_File *f;
+   unsigned char *seg = NULL, *filedata;
    tga_header *header;
    tga_footer *footer, tfooter;
-   char hasa = 0, footer_present = 0, vinverted = 0;
+   char hasa = 0;
    int w = 0, h = 0, bpp;
+   int x, y;
 
-   fd = open(file, O_RDONLY);
-   
+   f = eina_file_open(file, EINA_FALSE);
    *error = EVAS_LOAD_ERROR_DOES_NOT_EXIST;
-   if (fd < 0) return EINA_FALSE;
-   if (fstat(fd, &ss) < 0) goto close_file;
-   
+   if (f == NULL) return EINA_FALSE;
+
    *error = EVAS_LOAD_ERROR_UNKNOWN_FORMAT;
-   if (ss.st_size < (off_t)(sizeof(tga_header) + sizeof(tga_footer)))
+   if (eina_file_size_get(f) < (off_t)(sizeof(tga_header) + sizeof(tga_footer)))
       goto close_file;
-   seg = mmap(0, ss.st_size, PROT_READ, MAP_SHARED, fd, 0);
-   if (seg == MAP_FAILED) goto close_file;
+   seg = eina_file_map_all(f, EINA_FILE_SEQUENTIAL);
+   if (seg == NULL) goto close_file;
    filedata = seg;
-   
+
    header = (tga_header *)filedata;
    // no unaligned data accessed, so ok
-   footer = (tga_footer *)(filedata + (ss.st_size - sizeof(tga_footer)));
-   memcpy(&tfooter, footer, sizeof(tga_footer));
+   footer = (tga_footer *)(filedata + (eina_file_size_get(f) - sizeof(tga_footer)));
+   memcpy((unsigned char *)(&tfooter),
+          (unsigned char *)footer,
+          sizeof(tga_footer));
+   //printf("0\n");
    if (!memcmp(tfooter.signature, TGA_SIGNATURE, sizeof(tfooter.signature)))
      {
-        // footer is there and matches. this is a tga file - any problems now
-        // are a corrupt file
-        *error = EVAS_LOAD_ERROR_CORRUPT_FILE;
-        footer_present = 1;
+        if ((tfooter.dot == '.') && (tfooter.null == 0))
+          {
+             // footer is there and matches. this is a tga file - any problems now
+             // are a corrupt file
+             *error = EVAS_LOAD_ERROR_CORRUPT_FILE;
+          }
      }
+//   else goto close_file;
+   //printf("1\n");
+   
    filedata = (unsigned char *)filedata + sizeof(tga_header);
-   vinverted = !(header->descriptor & TGA_DESC_VERTICAL);
    switch (header->imageType)
      {
      case TGA_TYPE_COLOR_RLE:
@@ -121,8 +128,26 @@ evas_image_load_file_head_tga(Image_Entry *ie, const char *file, const char *key
    if (!((bpp == 32) || (bpp == 24) || (bpp == 16) || (bpp == 8)))
      goto close_file;
    if ((bpp == 32) && (header->descriptor & TGA_DESC_ABITS)) hasa = 1;
+   // don't handle colormapped images
+   if ((header->colorMapType) != 0)
+     goto close_file;
+   // if colormap size is anything other than legal sizes or 0 - not real tga
+   if (!((header->colorMapSize == 0) ||
+         (header->colorMapSize == 15) ||
+         (header->colorMapSize == 16) ||
+         (header->colorMapSize == 24) ||
+         (header->colorMapSize == 32)))
+     goto close_file;
+   x = (header->xOriginHi << 8) | (header->xOriginLo);
+   y = (header->yOriginHi << 8) | (header->yOriginLo);
    w = (header->widthHi << 8) | header->widthLo;
    h = (header->heightHi << 8) | header->heightLo;
+   // x origin gerater that width, y origin greater than height - wrong file
+   if ((x >= w) || (y >= h))
+     goto close_file;
+   // if descriptor has either of the top 2 bits set... not tga
+   if (header->descriptor & 0xc0)
+     goto close_file;
    
    if ((w < 1) || (h < 1) || (w > IMG_MAX_SIZE) || (h > IMG_MAX_SIZE) ||
        IMG_TOO_BIG(w, h))
@@ -131,56 +156,62 @@ evas_image_load_file_head_tga(Image_Entry *ie, const char *file, const char *key
    ie->w = w;
    ie->h = h;
    if (hasa) ie->flags.alpha = 1;
-   
-   if (seg != MAP_FAILED) munmap(seg, ss.st_size);
-   close(fd);
+
+   eina_file_map_free(f, seg);
+   eina_file_close(f);
    *error = EVAS_LOAD_ERROR_NONE;
    return EINA_TRUE;
 
 close_file:
-   if (seg != MAP_FAILED) munmap(seg, ss.st_size);
-   close(fd);
+   if (seg != NULL) eina_file_map_free(f, seg);
+   eina_file_close(f);
    return EINA_FALSE;
 }
 
 static Eina_Bool
 evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key __UNUSED__, int *error)
 {
-   int fd;
-   unsigned char *seg = MAP_FAILED, *filedata;
-   struct stat ss;
+   Eina_File *f;
+   unsigned char *seg = NULL, *filedata;
    tga_header *header;
    tga_footer *footer, tfooter;
    char hasa = 0, footer_present = 0, vinverted = 0, rle = 0;
    int w = 0, h = 0, x, y, bpp;
+   off_t size;
    unsigned int *surface, *dataptr;
    unsigned int  datasize;
    unsigned char *bufptr, *bufend;
-   
-   fd = open(file, O_RDONLY);
-   
+   int abits;
+
+   f = eina_file_open(file, EINA_FALSE);
    *error = EVAS_LOAD_ERROR_DOES_NOT_EXIST;
-   if (fd < 0) return EINA_FALSE;
-   if (fstat(fd, &ss) < 0) goto close_file;
-   
+   if (f == NULL) return EINA_FALSE;
+
    *error = EVAS_LOAD_ERROR_UNKNOWN_FORMAT;
-   if (ss.st_size < (off_t)(sizeof(tga_header) + sizeof(tga_footer)))
+   if (eina_file_size_get(f) < (off_t)(sizeof(tga_header) + sizeof(tga_footer)))
       goto close_file;
-   seg = mmap(0, ss.st_size, PROT_READ, MAP_SHARED, fd, 0);
-   if (seg == MAP_FAILED) goto close_file;
+   seg = eina_file_map_all(f, EINA_FILE_SEQUENTIAL);
+   if (seg == NULL) goto close_file;
    filedata = seg;
-   
+   size = eina_file_size_get(f);
+
    header = (tga_header *)filedata;
    // no unaligned data accessed, so ok
-   footer = (tga_footer *)(filedata + (ss.st_size - sizeof(tga_footer)));
-   memcpy(&tfooter, footer, sizeof(tga_footer));
+   footer = (tga_footer *)(filedata + (size - sizeof(tga_footer)));
+   memcpy((unsigned char *)&tfooter,
+          (unsigned char *)footer,
+          sizeof(tga_footer));
    if (!memcmp(tfooter.signature, TGA_SIGNATURE, sizeof(tfooter.signature)))
      {
-        // footer is there and matches. this is a tga file - any problems now
-        // are a corrupt file
-        *error = EVAS_LOAD_ERROR_CORRUPT_FILE;
-        footer_present = 1;
+        if ((tfooter.dot == '.') && (tfooter.null == 0))
+          {
+             // footer is there and matches. this is a tga file - any problems now
+             // are a corrupt file
+             *error = EVAS_LOAD_ERROR_CORRUPT_FILE;
+             footer_present = 1;
+          }
      }
+   
    filedata = (unsigned char *)filedata + sizeof(tga_header);
    vinverted = !(header->descriptor & TGA_DESC_VERTICAL);
    switch (header->imageType)
@@ -200,8 +231,27 @@ evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key
    if (!((bpp == 32) || (bpp == 24) || (bpp == 16) || (bpp == 8)))
      goto close_file;
    if ((bpp == 32) && (header->descriptor & TGA_DESC_ABITS)) hasa = 1;
+   abits = header->descriptor & TGA_DESC_ABITS;
+   // don't handle colormapped images
+   if ((header->colorMapType) != 0)
+     goto close_file;
+   // if colormap size is anything other than legal sizes or 0 - not real tga
+   if (!((header->colorMapSize == 0) ||
+         (header->colorMapSize == 15) ||
+         (header->colorMapSize == 16) ||
+         (header->colorMapSize == 24) ||
+         (header->colorMapSize == 32)))
+     goto close_file;
+   x = (header->xOriginHi << 8) | (header->xOriginLo);
+   y = (header->yOriginHi << 8) | (header->yOriginLo);
    w = (header->widthHi << 8) | header->widthLo;
    h = (header->heightHi << 8) | header->heightLo;
+   // x origin gerater that width, y origin greater than height - wrong file
+   if ((x >= w) || (y >= h))
+     goto close_file;
+   // if descriptor has either of the top 2 bits set... not tga
+   if (header->descriptor & 0xc0)
+     goto close_file;
    
    if ((w < 1) || (h < 1) || (w > IMG_MAX_SIZE) || (h > IMG_MAX_SIZE) ||
        IMG_TOO_BIG(w, h))
@@ -220,9 +270,9 @@ evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key
 	goto close_file;
      }
 
-   datasize = ss.st_size - sizeof(tga_header) - header->idLength;
+   datasize = size - sizeof(tga_header) - header->idLength;
    if (footer_present)
-     datasize = ss.st_size - sizeof(tga_header) - header->idLength - 
+     datasize = size - sizeof(tga_header) - header->idLength - 
      sizeof(tga_footer);
    
    bufptr = filedata + header->idLength;
@@ -243,7 +293,30 @@ evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key
                   for (x = 0; (x < w) && ((bufptr + 4) <= bufend); x++)
                     {
                        if (hasa)
-                         *dataptr = ARGB_JOIN(255 - bufptr[3], bufptr[2], bufptr[1], bufptr[0]);
+                         {
+                            int a = bufptr[3];
+                            
+                            switch (abits)
+                              {
+                               case 1:
+                                 a = (a << 7) | (a << 6) | (a << 5) | (a << 4) | (a << 3) | (a << 2) | (a << 1) | (a);
+                               case 2:
+                                 a = (a << 6) | (a << 4) | (a << 2) | (a);
+                               case 3:
+                                 a = (a << 5) | (a << 2) | (a >> 1);
+                               case 4:
+                                 a = (a << 4) | (a);
+                               case 5:
+                                 a = (a << 3) | (a >> 2);
+                               case 6:
+                                 a = (a << 2) | (a >> 4);
+                               case 7:
+                                 a = (a << 1) | (a >> 6);
+                               default:
+                                 break;
+                              }
+                            *dataptr = ARGB_JOIN(a, bufptr[2], bufptr[1], bufptr[0]);
+                         }
                        else
                          *dataptr = ARGB_JOIN(0xff, bufptr[2], bufptr[1], bufptr[0]);
                        dataptr++;
@@ -310,9 +383,28 @@ evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key
                     case 32:
                        if (bufptr < (bufend - 4))
                          {
-                            unsigned char r, g, b, a;
+                            unsigned char r, g, b;
+                            int a = bufptr[3];
                             
-                            a = 255 - bufptr[3];
+                            switch (abits)
+                              {
+                               case 1:
+                                 a = (a << 7) | (a << 6) | (a << 5) | (a << 4) | (a << 3) | (a << 2) | (a << 1) | (a);
+                               case 2:
+                                 a = (a << 6) | (a << 4) | (a << 2) | (a);
+                               case 3:
+                                 a = (a << 5) | (a << 2) | (a >> 1);
+                               case 4:
+                                 a = (a << 4) | (a);
+                               case 5:
+                                 a = (a << 3) | (a >> 2);
+                               case 6:
+                                 a = (a << 2) | (a >> 4);
+                               case 7:
+                                 a = (a << 1) | (a >> 6);
+                               default:
+                                 break;
+                              }
                             r = bufptr[2];
                             g = bufptr[1];
                             b = bufptr[0];
@@ -389,7 +481,8 @@ evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key
                        for (i = 0; (i < count) && (bufptr < (bufend - 4)) && (dataptr < dataend); i++)
                          {
                             if (hasa)
-                              *dataptr = ARGB_JOIN(255 - bufptr[3], bufptr[2], bufptr[1], bufptr[0]);
+//                              *dataptr = ARGB_JOIN(255 - bufptr[3], bufptr[2], bufptr[1], bufptr[0]);
+                              *dataptr = ARGB_JOIN(bufptr[3], bufptr[2], bufptr[1], bufptr[0]);
                             else
                               *dataptr = ARGB_JOIN(0xff, bufptr[2], bufptr[1], bufptr[0]);
                             dataptr++;
@@ -457,15 +550,15 @@ evas_image_load_file_data_tga(Image_Entry *ie, const char *file, const char *key
      }
    
    evas_common_image_premul(ie);
-   
-   if (seg != MAP_FAILED) munmap(seg, ss.st_size);
-   close(fd);
+
+   eina_file_map_free(f, seg);
+   eina_file_close(f);
    *error = EVAS_LOAD_ERROR_NONE;
    return EINA_TRUE;
 
 close_file:
-   if (seg != MAP_FAILED) munmap(seg, ss.st_size);
-   close(fd);
+   if (seg != NULL) eina_file_map_free(f, seg);
+   eina_file_close(f);
    return EINA_FALSE;
 }
 
