@@ -1023,7 +1023,7 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item absize=93x152 vsize=full></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w < 93) || (h != 152));
+   fail_if((w < 93) || (h < 153));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &h);
    fail_if((w != 93) || (h != 152));
@@ -1031,7 +1031,7 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item absize=93x152 vsize=ascent></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w < 93) || (h <= 152));
+   fail_if((w < 93) || (h <= 153));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &h);
    fail_if((w != 93) || (h != 152));
@@ -1040,7 +1040,7 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item size=93x152 vsize=full></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w < 93) || (h != 152));
+   fail_if((w < 93) || (h < 153));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &h);
    fail_if((w != 93) || (h != 152));
@@ -1048,7 +1048,7 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item size=93x152 vsize=ascent></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w < 93) || (h <= 152));
+   fail_if((w < 93) || (h <= 153));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &h);
    fail_if((w != 93) || (h != 152));
@@ -1057,7 +1057,7 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item size=93x152 vsize=full></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w < (2 * 93)) || (h != (2 * 152)));
+   fail_if((w < (2 * 93)) || (h < (2 * 154)));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &h);
    fail_if((w != (2 * 93)) || (h != (2 * 152)));
@@ -1068,7 +1068,7 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item size=93x152 vsize=ascent></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w < (2 * 93)) || (h <= (2 * 152)));
+   fail_if((w < (2 * 93)) || (h <= (2 * 154)));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &h);
    fail_if((w != (2 * 93)) || (h != (2 * 152)));
@@ -1080,10 +1080,10 @@ START_TEST(evas_textblock_items)
    buf = "This is an <item relsize=93x152 vsize=full></>.";
    evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
-   fail_if((w >= 93) || (h >= 152));
+   fail_if((w >= 93) || (h >= 153));
    evas_textblock_cursor_pos_set(cur, 11);
    evas_textblock_cursor_format_item_geometry_get(cur, NULL, NULL, &w, &ih);
-   fail_if((w > 90) || (h != ih));
+   fail_if((w > 90) || (h <= ih));
 
    buf = "This is an <item relize=93x152 vsize=ascent></>.";
    evas_object_textblock_text_markup_set(tb, buf);
@@ -2183,11 +2183,11 @@ START_TEST(evas_textblock_escaping)
 
    const char *buf = "This &middot; is";
    evas_object_textblock_text_markup_set(tb, buf);
-   fail_if(strcmp(evas_object_textblock_text_markup_get(tb), buf));
+   fail_if(strcmp(evas_object_textblock_text_markup_get(tb), "This \xc2\xb7 is"));
 
    buf = "This &nbsp; is";
    evas_object_textblock_text_markup_set(tb, buf);
-   fail_if(strcmp(evas_object_textblock_text_markup_get(tb), buf));
+   fail_if(strcmp(evas_object_textblock_text_markup_get(tb), "This \xc2\xa0 is"));
 
    END_TB_TEST();
 }
@@ -2198,21 +2198,30 @@ START_TEST(evas_textblock_size)
    START_TB_TEST();
    Evas_Coord w, h, h2, nw, nh;
    const char *buf = "This is a <br/> test.<br/>גם בעברית";
+
+   /* Empty textblock */
+   evas_object_textblock_size_formatted_get(tb, &w, &h);
+   evas_object_textblock_size_native_get(tb, &nw, &nh);
+   ck_assert_int_eq(w, nw);
+   ck_assert_int_eq(h, nh);
+   fail_if(w != 0);
+
+
    /* When wrapping is off, native size should be the same as formatted
     * size */
 
+   evas_object_textblock_text_markup_set(tb, buf);
    evas_object_textblock_size_formatted_get(tb, &w, &h);
    evas_object_textblock_size_native_get(tb, &nw, &nh);
    fail_if((w != nw) || (h != nh));
-   fail_if(w != 0);
 
    evas_object_textblock_text_markup_set(tb, "a<br/>a");
    evas_object_textblock_size_formatted_get(tb, &w, &h2);
    evas_object_textblock_size_native_get(tb, &nw, &nh);
    fail_if((w != nw) || (h2 != nh));
 
-   /* Two lines == double the height */
-   fail_if(h * 2 != h2);
+   /* Two lines != double the height */
+   fail_if(h * 2 == h2);
 
    evas_object_textblock_text_markup_set(tb, buf);
 
